@@ -3,15 +3,11 @@
 # lib/matplotlib/backends/web_backend/nbagg_uat.ipynb to help verify
 # that changes made maintain expected behaviour.
 
-import datetime
 from base64 import b64encode
-import json
 import io
-import os
-from matplotlib.externals import six
-from uuid import uuid4 as uuid
-
-import tornado.ioloop
+import json
+import pathlib
+import uuid
 
 from IPython.display import display, Javascript, HTML
 try:
@@ -21,68 +17,30 @@ except ImportError:
     # Jupyter/IPython 3.x or earlier
     from IPython.kernel.comm import Comm
 
-from matplotlib import rcParams
-from matplotlib.figure import Figure
-from matplotlib import is_interactive
-from matplotlib.backends.backend_webagg_core import (FigureManagerWebAgg,
-                                                     FigureCanvasWebAggCore,
-                                                     NavigationToolbar2WebAgg,
-                                                     TimerTornado)
-from matplotlib.backend_bases import (ShowBase, NavigationToolbar2,
-                                      FigureCanvasBase)
-
-
-class Show(ShowBase):
-    def __call__(self, block=None):
-        from matplotlib._pylab_helpers import Gcf
-
-        managers = Gcf.get_all_fig_managers()
-        if not managers:
-            return
-
-        interactive = is_interactive()
-
-        for manager in managers:
-            manager.show()
-
-            # plt.figure adds an event which puts the figure in focus
-            # in the activeQue. Disable this behaviour, as it results in
-            # figures being put as the active figure after they have been
-            # shown, even in non-interactive mode.
-            if hasattr(manager, '_cidgcf'):
-                manager.canvas.mpl_disconnect(manager._cidgcf)
-
-            if not interactive and manager in Gcf._activeQue:
-                Gcf._activeQue.remove(manager)
-
-show = Show()
-
-
-def draw_if_interactive():
-    import matplotlib._pylab_helpers as pylab_helpers
-
-    if is_interactive():
-        manager = pylab_helpers.Gcf.get_active()
-        if manager is not None:
-            manager.show()
+from matplotlib import cbook, is_interactive
+from matplotlib._pylab_helpers import Gcf
+from matplotlib.backend_bases import (
+    _Backend, FigureCanvasBase, NavigationToolbar2)
+from matplotlib.backends.backend_webagg_core import (
+    FigureCanvasWebAggCore, FigureManagerWebAgg, NavigationToolbar2WebAgg,
+    TimerTornado)
 
 
 def connection_info():
     """
-    Return a string showing the figure and connection status for
-    the backend. This is intended as a diagnostic tool, and not for general
-    use.
+    Return a string showing the figure and connection status for the backend.
 
+    This is intended as a diagnostic tool, and not for general use.
     """
-    from matplotlib._pylab_helpers import Gcf
-    result = []
-    for manager in Gcf.get_all_fig_managers():
-        fig = manager.canvas.figure
-        result.append('{0} - {0}'.format((fig.get_label() or
-                                          "Figure {0}".format(manager.num)),
-                                         manager.web_sockets))
+    result = [
+        '{fig} - {socket}'.format(
+            fig=(manager.canvas.figure.get_label()
+                 or "Figure {}".format(manager.num)),
+            socket=manager.web_sockets)
+        for manager in Gcf.get_all_fig_managers()
+    ]
     if not is_interactive():
-        result.append('Figures pending show: {0}'.format(len(Gcf._activeQue)))
+        result.append(f'Figures pending show: {len(Gcf.figs)}')
     return '\n'.join(result)
 
 
@@ -152,12 +110,10 @@ class FigureManagerNbAgg(FigureManagerWebAgg):
             output = io.StringIO()
         else:
             output = stream
-        super(FigureManagerNbAgg, cls).get_javascript(stream=output)
-        with io.open(os.path.join(
-                os.path.dirname(__file__),
-                "web_backend",
-                "nbagg_mpl.js"), encoding='utf8') as fd:
-            output.write(fd.read())
+        super().get_javascript(stream=output)
+        output.write((pathlib.Path(__file__).parent
+                      / "web_backend/js/nbagg_mpl.js")
+                     .read_text(encoding="utf-8"))
         if stream is None:
             return output.getvalue()
 
@@ -175,61 +131,24 @@ class FigureManagerNbAgg(FigureManagerWebAgg):
 
     def clearup_closed(self):
         """Clear up any closed Comms."""
-        self.web_sockets = set([socket for socket in self.web_sockets
-                                if socket.is_open()])
+        self.web_sockets = {socket for socket in self.web_sockets
+                            if socket.is_open()}
 
         if len(self.web_sockets) == 0:
             self.canvas.close_event()
 
     def remove_comm(self, comm_id):
-        self.web_sockets = set([socket for socket in self.web_sockets
-                                if not socket.comm.comm_id == comm_id])
+        self.web_sockets = {socket for socket in self.web_sockets
+                            if socket.comm.comm_id != comm_id}
 
 
 class FigureCanvasNbAgg(FigureCanvasWebAggCore):
     def new_timer(self, *args, **kwargs):
+        # docstring inherited
         return TimerTornado(*args, **kwargs)
 
-    def start_event_loop(self, timeout):
-        FigureCanvasBase.start_event_loop_default(self, timeout)
 
-    def stop_event_loop(self):
-        FigureCanvasBase.stop_event_loop_default(self)
-
-
-def new_figure_manager(num, *args, **kwargs):
-    """
-    Create a new figure manager instance
-    """
-    FigureClass = kwargs.pop('FigureClass', Figure)
-    thisFig = FigureClass(*args, **kwargs)
-    return new_figure_manager_given_figure(num, thisFig)
-
-
-def new_figure_manager_given_figure(num, figure):
-    """
-    Create a new figure manager instance for the given figure.
-    """
-    from .._pylab_helpers import Gcf
-
-    def closer(event):
-        Gcf.destroy(num)
-
-    canvas = FigureCanvasNbAgg(figure)
-    if rcParams['nbagg.transparent']:
-        figure.patch.set_alpha(0)
-    manager = FigureManagerNbAgg(canvas, num)
-
-    if is_interactive():
-        manager.show()
-        figure.canvas.draw_idle()
-
-    canvas.mpl_connect('close_event', closer)
-
-    return manager
-
-
-class CommSocket(object):
+class CommSocket:
     """
     Manages the Comm connection between IPython and the browser (client).
 
@@ -242,7 +161,7 @@ class CommSocket(object):
     def __init__(self, manager):
         self.supports_binary = None
         self.manager = manager
-        self.uuid = str(uuid())
+        self.uuid = str(uuid.uuid4())
         # Publish an output area with a unique ID. The javascript can then
         # hook into this area.
         display(HTML("<div id=%r></div>" % self.uuid))
@@ -282,9 +201,7 @@ class CommSocket(object):
     def send_binary(self, blob):
         # The comm is ascii, so we always send the image in base64
         # encoded data URL form.
-        data = b64encode(blob)
-        if six.PY3:
-            data = data.decode('ascii')
+        data = b64encode(blob).decode('ascii')
         data_uri = "data:image/png;base64,{0}".format(data)
         self.comm.send({'data': data_uri})
 
@@ -302,3 +219,53 @@ class CommSocket(object):
             self.supports_binary = message['value']
         else:
             self.manager.handle_json(message)
+
+
+@_Backend.export
+class _BackendNbAgg(_Backend):
+    FigureCanvas = FigureCanvasNbAgg
+    FigureManager = FigureManagerNbAgg
+
+    @staticmethod
+    def new_figure_manager_given_figure(num, figure):
+        canvas = FigureCanvasNbAgg(figure)
+        manager = FigureManagerNbAgg(canvas, num)
+        if is_interactive():
+            manager.show()
+            figure.canvas.draw_idle()
+        canvas.mpl_connect('close_event', lambda event: Gcf.destroy(num))
+        return manager
+
+    @staticmethod
+    def trigger_manager_draw(manager):
+        manager.show()
+
+    @staticmethod
+    def show(*args, block=None, **kwargs):
+        if args or kwargs:
+            cbook.warn_deprecated(
+                "3.1", message="Passing arguments to show(), other than "
+                "passing 'block' by keyword, is deprecated %(since)s, and "
+                "support for it will be removed %(removal)s.")
+
+        ## TODO: something to do when keyword block==False ?
+        from matplotlib._pylab_helpers import Gcf
+
+        managers = Gcf.get_all_fig_managers()
+        if not managers:
+            return
+
+        interactive = is_interactive()
+
+        for manager in managers:
+            manager.show()
+
+            # plt.figure adds an event which makes the figure in focus the
+            # active one. Disable this behaviour, as it results in
+            # figures being put as the active figure after they have been
+            # shown, even in non-interactive mode.
+            if hasattr(manager, '_cidgcf'):
+                manager.canvas.mpl_disconnect(manager._cidgcf)
+
+            if not interactive:
+                Gcf.figs.pop(manager.num, None)
