@@ -1,46 +1,44 @@
 """
-This module provides a large set of colormaps, functions for
-registering new colormaps and for getting a colormap by name,
-and a mixin class for adding color mapping functionality.
+Builtin colormaps, colormap handling utilities, and the `ScalarMappable` mixin.
 
+.. seealso::
+
+  :doc:`/gallery/color/colormap_reference` for a list of builtin colormaps.
+
+  :doc:`/tutorials/colors/colormap-manipulation` for examples of how to
+  make colormaps.
+
+  :doc:`/tutorials/colors/colormaps` an in-depth discussion of
+  choosing colormaps.
+
+  :doc:`/tutorials/colors/colormapnorms` for more details about data
+  normalization.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-import six
+import functools
 
-import os
 import numpy as np
 from numpy import ma
+
 import matplotlib as mpl
 import matplotlib.colors as colors
 import matplotlib.cbook as cbook
-from matplotlib._cm import datad, _deprecation_datad
-from matplotlib._cm import cubehelix
+from matplotlib._cm import datad
 from matplotlib._cm_listed import cmaps as cmaps_listed
 
-cmap_d = _deprecation_datad()
 
-# reverse all the colormaps.
-# reversed colormaps have '_r' appended to the name.
-
-
-def _reverser(f):
-    def freversed(x):
-        return f(1 - x)
-    return freversed
+def _reverser(f, x):  # Deprecated, remove this at the same time as revcmap.
+    return f(1 - x)  # Toplevel helper for revcmap ensuring cmap picklability.
 
 
+@cbook.deprecated("3.2", alternative="Colormap.reversed()")
 def revcmap(data):
     """Can only handle specification *data* in dictionary format."""
     data_r = {}
-    for key, val in six.iteritems(data):
-        if six.callable(val):
-            valnew = _reverser(val)
-            # This doesn't work: lambda x: val(1-x)
-            # The same "val" (the first one) is used
-            # each time, so the colors are identical
-            # and the result is shades of gray.
+    for key, val in data.items():
+        if callable(val):
+            # Return a partial object so that the result is picklable.
+            valnew = functools.partial(_reverser, val)
         else:
             # Flip x and exchange the y values facing x = 0 and x = 1.
             valnew = [(1.0 - x, y1, y0) for x, y0, y1 in reversed(val)]
@@ -48,55 +46,30 @@ def revcmap(data):
     return data_r
 
 
-def _reverse_cmap_spec(spec):
-    """Reverses cmap specification *spec*, can handle both dict and tuple
-    type specs."""
-
-    if 'listed' in spec:
-        return {'listed': spec['listed'][::-1]}
-
-    if 'red' in spec:
-        return revcmap(spec)
-    else:
-        revspec = list(reversed(spec))
-        if len(revspec[0]) == 2:    # e.g., (1, (1.0, 0.0, 1.0))
-            revspec = [(1.0 - a, b) for a, b in revspec]
-        return revspec
-
-
-def _generate_cmap(name, lutsize):
-    """Generates the requested cmap from its *name*.  The lut size is
-    *lutsize*."""
-
-    # Use superclass method to avoid deprecation warnings during initial load.
-    spec = dict.__getitem__(datad, name)
-
-    # Generate the colormap object.
-    if 'red' in spec:
-        return colors.LinearSegmentedColormap(name, spec, lutsize)
-    elif 'listed' in spec:
-        return colors.ListedColormap(spec['listed'], name)
-    else:
-        return colors.LinearSegmentedColormap.from_list(name, spec, lutsize)
-
 LUTSIZE = mpl.rcParams['image.lut']
 
-# Generate the reversed specifications ...
-for cmapname in list(six.iterkeys(datad)):
-    # Use superclass method to avoid deprecation warnings during initial load.
-    spec = dict.__getitem__(datad, cmapname)
-    spec_reversed = _reverse_cmap_spec(spec)
-    datad[cmapname + '_r'] = spec_reversed
 
-# Precache the cmaps with ``lutsize = LUTSIZE`` ...
+def _gen_cmap_d():
+    """
+    Generate a dict mapping standard colormap names to standard colormaps, as
+    well as the reversed colormaps.
+    """
+    cmap_d = {**cmaps_listed}
+    for name, spec in datad.items():
+        cmap_d[name] = (  # Precache the cmaps at a fixed lutsize..
+            colors.LinearSegmentedColormap(name, spec, LUTSIZE)
+            if 'red' in spec else
+            colors.ListedColormap(spec['listed'], name)
+            if 'listed' in spec else
+            colors.LinearSegmentedColormap.from_list(name, spec, LUTSIZE))
+    # Generate reversed cmaps.
+    for cmap in list(cmap_d.values()):
+        rmap = cmap.reversed()
+        cmap_d[rmap.name] = rmap
+    return cmap_d
 
-# Use datad.keys() to also add the reversed ones added in the section
-# above:
-for cmapname in six.iterkeys(datad):
-    cmap_d[cmapname] = _generate_cmap(cmapname, LUTSIZE)
-
-cmap_d.update(cmaps_listed)
 
+cmap_d = _gen_cmap_d()
 locals().update(cmap_d)
 
 
@@ -120,21 +93,16 @@ def register_cmap(name=None, cmap=None, data=None, lut=None):
     In the second case, the three arguments are passed to
     the :class:`~matplotlib.colors.LinearSegmentedColormap` initializer,
     and the resulting colormap is registered.
-
     """
+    cbook._check_isinstance((str, None), name=name)
     if name is None:
         try:
             name = cmap.name
         except AttributeError:
             raise ValueError("Arguments must include a name or a Colormap")
-
-    if not cbook.is_string_like(name):
-        raise ValueError("Colormap name must be a string")
-
     if isinstance(cmap, colors.Colormap):
         cmap_d[name] = cmap
         return
-
     # For the remainder, let exceptions propagate.
     if lut is None:
         lut = mpl.rcParams['image.lut']
@@ -149,31 +117,28 @@ def get_cmap(name=None, lut=None):
     Colormaps added with :func:`register_cmap` take precedence over
     built-in colormaps.
 
-    If *name* is a :class:`matplotlib.colors.Colormap` instance, it will be
-    returned.
-
-    If *lut* is not None it must be an integer giving the number of
-    entries desired in the lookup table, and *name* must be a standard
-    mpl colormap name.
+    Parameters
+    ----------
+    name : `matplotlib.colors.Colormap` or str or None, default: None
+        If a `Colormap` instance, it will be returned.  Otherwise, the name of
+        a colormap known to Matplotlib, which will be resampled by *lut*.  The
+        default, None, means :rc:`image.cmap`.
+    lut : int or None, default: None
+        If *name* is not already a Colormap instance and *lut* is not None, the
+        colormap will be resampled to have *lut* entries in the lookup table.
     """
     if name is None:
         name = mpl.rcParams['image.cmap']
-
     if isinstance(name, colors.Colormap):
         return name
-
-    if name in cmap_d:
-        if lut is None:
-            return cmap_d[name]
-        else:
-            return cmap_d[name]._resample(lut)
+    cbook._check_in_list(sorted(cmap_d), name=name)
+    if lut is None:
+        return cmap_d[name]
     else:
-        raise ValueError(
-            "Colormap %s is not recognized. Possible values are: %s"
-            % (name, ', '.join(sorted(cmap_d.keys()))))
+        return cmap_d[name]._resample(lut)
 
 
-class ScalarMappable(object):
+class ScalarMappable:
     """
     This is a mixin class to support scalar data to RGBA mapping.
     The ScalarMappable makes use of data normalization before returning
@@ -181,7 +146,7 @@ class ScalarMappable(object):
 
     """
     def __init__(self, norm=None, cmap=None):
-        r"""
+        """
 
         Parameters
         ----------
@@ -223,6 +188,9 @@ class ScalarMappable(object):
         If *x* is an ndarray with 3 dimensions,
         and the last dimension is either 3 or 4, then it will be
         treated as an rgb or rgba array, and no mapping will be done.
+        The array can be uint8, or it can be floating point with
+        values in the 0-1 range; otherwise a ValueError will be raised.
+        If it is a masked array, the mask will be ignored.
         If the last dimension is 3, the *alpha* kwarg (defaulting to 1)
         will be used to fill in the transparency.  If the last dimension
         is 4, the *alpha* kwarg is ignored; it does not
@@ -234,12 +202,8 @@ class ScalarMappable(object):
         the returned rgba array will be uint8 in the 0 to 255 range.
 
         If norm is False, no normalization of the input data is
-        performed, and it is assumed to already be in the range (0-1).
+        performed, and it is assumed to be in the range (0-1).
 
-        Note: this method assumes the input is well-behaved; it does
-        not check for anomalies such as *x* being a masked rgba
-        array, or being an integer type other than uint8, or being
-        a floating point rgba array with values outside the 0-1 range.
         """
         # First check for special case, image input:
         try:
@@ -257,10 +221,18 @@ class ScalarMappable(object):
                     xx = x
                 else:
                     raise ValueError("third dimension must be 3 or 4")
-                if bytes and xx.dtype != np.uint8:
-                    xx = (xx * 255).astype(np.uint8)
-                if not bytes and xx.dtype == np.uint8:
-                    xx = xx.astype(float) / 255
+                if xx.dtype.kind == 'f':
+                    if norm and (xx.max() > 1 or xx.min() < 0):
+                        raise ValueError("Floating point image RGB values "
+                                         "must be in the 0..1 range.")
+                    if bytes:
+                        xx = (xx * 255).astype(np.uint8)
+                elif xx.dtype == np.uint8:
+                    if not bytes:
+                        xx = xx.astype(np.float32) / 255
+                else:
+                    raise ValueError("Image RGB array must be uint8 or "
+                                     "floating point; found %s" % xx.dtype)
                 return xx
         except AttributeError:
             # e.g., x is not an ndarray; so try mapping it
@@ -274,7 +246,12 @@ class ScalarMappable(object):
         return rgba
 
     def set_array(self, A):
-        'Set the image array from numpy array *A*'
+        """Set the image array from numpy array *A*.
+
+        Parameters
+        ----------
+        A : ndarray
+        """
         self._A = A
         self.update_dict['array'] = True
 
@@ -292,34 +269,66 @@ class ScalarMappable(object):
 
     def set_clim(self, vmin=None, vmax=None):
         """
-        set the norm limits for image scaling; if *vmin* is a length2
-        sequence, interpret it as ``(vmin, vmax)`` which is used to
-        support setp
+        Set the norm limits for image scaling.
 
-        ACCEPTS: a length 2 sequence of floats
+        Parameters
+        ----------
+        vmin, vmax : float
+             The limits.
+
+             The limits may also be passed as a tuple (*vmin*, *vmax*) as a
+             single positional argument.
+
+             .. ACCEPTS: (vmin: float, vmax: float)
         """
-        if (vmin is not None and vmax is None and
-                cbook.iterable(vmin) and len(vmin) == 2):
-            vmin, vmax = vmin
-
+        if vmax is None:
+            try:
+                vmin, vmax = vmin
+            except (TypeError, ValueError):
+                pass
         if vmin is not None:
-            self.norm.vmin = vmin
+            self.norm.vmin = colors._sanitize_extrema(vmin)
         if vmax is not None:
-            self.norm.vmax = vmax
+            self.norm.vmax = colors._sanitize_extrema(vmax)
         self.changed()
 
+    def get_alpha(self):
+        """
+        Returns
+        -------
+        alpha : float
+            Always returns 1.
+        """
+        # This method is intended to be overridden by Artist sub-classes
+        return 1.
+
     def set_cmap(self, cmap):
         """
         set the colormap for luminance data
 
-        ACCEPTS: a colormap or registered colormap name
+        Parameters
+        ----------
+        cmap : colormap or registered colormap name
         """
         cmap = get_cmap(cmap)
         self.cmap = cmap
         self.changed()
 
     def set_norm(self, norm):
-        'set the normalization instance'
+        """Set the normalization instance.
+
+        Parameters
+        ----------
+        norm : `.Normalize`
+
+        Notes
+        -----
+        If there are any colorbars using the mappable for this norm, setting
+        the norm of the mappable will reset the norm, locator, and formatters
+        on the colorbar to default.
+
+        """
+        cbook._check_isinstance((colors.Normalize, None), norm=norm)
         if norm is None:
             norm = colors.Normalize()
         self.norm = norm
@@ -371,3 +380,4 @@ class ScalarMappable(object):
 
         for key in self.update_dict:
             self.update_dict[key] = True
+        self.stale = True
