@@ -22,8 +22,7 @@ class GeoAxes(Axes):
             self._round_to = round_to
 
         def __call__(self, x, pos=None):
-            degrees = (x / np.pi) * 180.0
-            degrees = np.round(degrees / self._round_to) * self._round_to
+            degrees = round(np.rad2deg(x) / self._round_to) * self._round_to
             if rcParams['text.usetex'] and not rcParams['text.latex.unicode']:
                 return r"$%0.0f^\circ$" % degrees
             else:
@@ -99,8 +98,8 @@ class GeoAxes(Axes):
         yaxis_text_base = \
             yaxis_stretch + \
             self.transProjection + \
-            (yaxis_space + \
-             self.transAffine + \
+            (yaxis_space +
+             self.transAffine +
              self.transAxes)
         self._yaxis_text1_transform = \
             yaxis_text_base + \
@@ -111,8 +110,8 @@ class GeoAxes(Axes):
 
     def _get_affine_transform(self):
         transform = self._get_core_transform(1)
-        xscale, _ = transform.transform_point((np.pi, 0))
-        _, yscale = transform.transform_point((0, np.pi / 2))
+        xscale, _ = transform.transform((np.pi, 0))
+        _, yscale = transform.transform((0, np.pi/2))
         return Affine2D() \
             .scale(0.5 / xscale, 0.5 / yscale) \
             .translate(0.5, 0.5)
@@ -214,7 +213,7 @@ class GeoAxes(Axes):
         """
         return False
 
-    def can_pan(self) :
+    def can_pan(self):
         """
         Return *True* if this axes supports the pan/zoom button functionality.
 
@@ -265,8 +264,7 @@ class AitoffAxes(GeoAxes):
 
         def transform_non_affine(self, ll):
             # docstring inherited
-            longitude = ll[:, 0]
-            latitude = ll[:, 1]
+            longitude, latitude = ll.T
 
             # Pre-compute some values
             half_long = longitude / 2.0
@@ -278,10 +276,9 @@ class AitoffAxes(GeoAxes):
             # We want unnormalized sinc.  numpy.sinc gives us normalized
             sinc_alpha = np.sin(alpha) / alpha
 
-            xy = np.empty_like(ll, float)
-            xy[:, 0] = (cos_latitude * np.sin(half_long)) / sinc_alpha
-            xy[:, 1] = np.sin(latitude) / sinc_alpha
-            return xy
+            x = (cos_latitude * np.sin(half_long)) / sinc_alpha
+            y = np.sin(latitude) / sinc_alpha
+            return np.column_stack([x, y])
 
         def inverted(self):
             # docstring inherited
@@ -316,18 +313,14 @@ class HammerAxes(GeoAxes):
 
         def transform_non_affine(self, ll):
             # docstring inherited
-            longitude = ll[:, 0:1]
-            latitude  = ll[:, 1:2]
-
-            # Pre-compute some values
+            longitude, latitude = ll.T
             half_long = longitude / 2.0
             cos_latitude = np.cos(latitude)
             sqrt2 = np.sqrt(2.0)
-
             alpha = np.sqrt(1.0 + cos_latitude * np.cos(half_long))
             x = (2.0 * sqrt2) * (cos_latitude * np.sin(half_long)) / alpha
             y = (sqrt2 * np.sin(latitude)) / alpha
-            return np.concatenate((x, y), 1)
+            return np.column_stack([x, y])
 
         def inverted(self):
             # docstring inherited
@@ -370,8 +363,7 @@ class MollweideAxes(GeoAxes):
                          / (1 + np.cos(theta)))
                 return delta, np.abs(delta) > 0.001
 
-            longitude = ll[:, 0]
-            latitude  = ll[:, 1]
+            longitude, latitude = ll.T
 
             clat = np.pi/2 - np.abs(latitude)
             ihigh = clat < 0.087  # within 5 degrees of the poles
@@ -406,16 +398,13 @@ class MollweideAxes(GeoAxes):
 
         def transform_non_affine(self, xy):
             # docstring inherited
-            x = xy[:, 0:1]
-            y = xy[:, 1:2]
-
+            x, y = xy.T
             # from Equations (7, 8) of
             # http://mathworld.wolfram.com/MollweideProjection.html
             theta = np.arcsin(y / np.sqrt(2))
-            lon = (np.pi / (2 * np.sqrt(2))) * x / np.cos(theta)
-            lat = np.arcsin((2 * theta + np.sin(2 * theta)) / np.pi)
-
-            return np.concatenate((lon, lat), 1)
+            longitude = (np.pi / (2 * np.sqrt(2))) * x / np.cos(theta)
+            latitude = np.arcsin((2 * theta + np.sin(2 * theta)) / np.pi)
+            return np.column_stack([longitude, latitude])
 
         def inverted(self):
             # docstring inherited
@@ -449,8 +438,7 @@ class LambertAxes(GeoAxes):
 
         def transform_non_affine(self, ll):
             # docstring inherited
-            longitude = ll[:, 0:1]
-            latitude  = ll[:, 1:2]
+            longitude, latitude = ll.T
             clong = self._center_longitude
             clat = self._center_latitude
             cos_lat = np.cos(latitude)
@@ -465,7 +453,7 @@ class LambertAxes(GeoAxes):
             x = k * cos_lat*np.sin(diff_long)
             y = k * (np.cos(clat)*sin_lat - np.sin(clat)*cos_lat*cos_diff_long)
 
-            return np.concatenate((x, y), 1)
+            return np.column_stack([x, y])
 
         def inverted(self):
             # docstring inherited
@@ -483,8 +471,7 @@ class LambertAxes(GeoAxes):
 
         def transform_non_affine(self, xy):
             # docstring inherited
-            x = xy[:, 0:1]
-            y = xy[:, 1:2]
+            x, y = xy.T
             clong = self._center_longitude
             clat = self._center_latitude
             p = np.maximum(np.hypot(x, y), 1e-9)
@@ -492,12 +479,12 @@ class LambertAxes(GeoAxes):
             sin_c = np.sin(c)
             cos_c = np.cos(c)
 
-            lat = np.arcsin(cos_c*np.sin(clat) +
-                             ((y*sin_c*np.cos(clat)) / p))
-            lon = clong + np.arctan(
+            latitude = np.arcsin(cos_c*np.sin(clat) +
+                                 ((y*sin_c*np.cos(clat)) / p))
+            longitude = clong + np.arctan(
                 (x*sin_c) / (p*np.cos(clat)*cos_c - y*np.sin(clat)*sin_c))
 
-            return np.concatenate((lon, lat), 1)
+            return np.column_stack([longitude, latitude])
 
         def inverted(self):
             # docstring inherited
