@@ -1,13 +1,11 @@
 """
 Tests specific to the collections module.
 """
-from __future__ import absolute_import, division, print_function
-
 import io
+import platform
 
 import numpy as np
-from numpy.testing import (
-    assert_array_equal, assert_array_almost_equal, assert_equal)
+from numpy.testing import assert_array_equal, assert_array_almost_equal
 import pytest
 
 import matplotlib.pyplot as plt
@@ -60,7 +58,7 @@ def generate_EventCollection_plot():
     return splt, coll, props
 
 
-@image_comparison(baseline_images=['EventCollection_plot__default'])
+@image_comparison(['EventCollection_plot__default'])
 def test__EventCollection__get_segments():
     '''
     check to make sure the default segments have the correct coordinates
@@ -87,7 +85,7 @@ def test__EventCollection__get_orientation():
     orientation
     '''
     _, coll, props = generate_EventCollection_plot()
-    assert_equal(props['orientation'], coll.get_orientation())
+    assert props['orientation'] == coll.get_orientation()
 
 
 def test__EventCollection__is_horizontal():
@@ -96,7 +94,7 @@ def test__EventCollection__is_horizontal():
     orientation
     '''
     _, coll, _ = generate_EventCollection_plot()
-    assert_equal(True, coll.is_horizontal())
+    assert coll.is_horizontal()
 
 
 def test__EventCollection__get_linelength():
@@ -104,7 +102,7 @@ def test__EventCollection__get_linelength():
     check to make sure the default linelength matches the input linelength
     '''
     _, coll, props = generate_EventCollection_plot()
-    assert_equal(props['linelength'], coll.get_linelength())
+    assert props['linelength'] == coll.get_linelength()
 
 
 def test__EventCollection__get_lineoffset():
@@ -112,7 +110,7 @@ def test__EventCollection__get_lineoffset():
     check to make sure the default lineoffset matches the input lineoffset
     '''
     _, coll, props = generate_EventCollection_plot()
-    assert_equal(props['lineoffset'], coll.get_lineoffset())
+    assert props['lineoffset'] == coll.get_lineoffset()
 
 
 def test__EventCollection__get_linestyle():
@@ -120,7 +118,7 @@ def test__EventCollection__get_linestyle():
     check to make sure the default linestyle matches the input linestyle
     '''
     _, coll, _ = generate_EventCollection_plot()
-    assert_equal(coll.get_linestyle(), [(None, None)])
+    assert coll.get_linestyle() == [(None, None)]
 
 
 def test__EventCollection__get_color():
@@ -132,7 +130,7 @@ def test__EventCollection__get_color():
     check_allprop_array(coll.get_colors(), props['color'])
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_positions'])
+@image_comparison(['EventCollection_plot__set_positions'])
 def test__EventCollection__set_positions():
     '''
     check to make sure set_positions works properly
@@ -149,7 +147,7 @@ def test__EventCollection__set_positions():
     splt.set_xlim(-1, 90)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__add_positions'])
+@image_comparison(['EventCollection_plot__add_positions'])
 def test__EventCollection__add_positions():
     '''
     check to make sure add_positions works properly
@@ -168,7 +166,7 @@ def test__EventCollection__add_positions():
     splt.set_xlim(-1, 35)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__append_positions'])
+@image_comparison(['EventCollection_plot__append_positions'])
 def test__EventCollection__append_positions():
     '''
     check to make sure append_positions works properly
@@ -187,7 +185,7 @@ def test__EventCollection__append_positions():
     splt.set_xlim(-1, 90)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__extend_positions'])
+@image_comparison(['EventCollection_plot__extend_positions'])
 def test__EventCollection__extend_positions():
     '''
     check to make sure extend_positions works properly
@@ -206,7 +204,7 @@ def test__EventCollection__extend_positions():
     splt.set_xlim(-1, 90)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__switch_orientation'])
+@image_comparison(['EventCollection_plot__switch_orientation'])
 def test__EventCollection__switch_orientation():
     '''
     check to make sure switch_orientation works properly
@@ -214,8 +212,8 @@ def test__EventCollection__switch_orientation():
     splt, coll, props = generate_EventCollection_plot()
     new_orientation = 'vertical'
     coll.switch_orientation()
-    assert_equal(new_orientation, coll.get_orientation())
-    assert_equal(False, coll.is_horizontal())
+    assert new_orientation == coll.get_orientation()
+    assert not coll.is_horizontal()
     new_positions = coll.get_positions()
     check_segments(coll,
                    new_positions,
@@ -226,8 +224,7 @@ def test__EventCollection__switch_orientation():
     splt.set_xlim(0, 2)
 
 
-@image_comparison(
-    baseline_images=['EventCollection_plot__switch_orientation__2x'])
+@image_comparison(['EventCollection_plot__switch_orientation__2x'])
 def test__EventCollection__switch_orientation_2x():
     '''
     check to make sure calling switch_orientation twice sets the
@@ -237,8 +234,8 @@ def test__EventCollection__switch_orientation_2x():
     coll.switch_orientation()
     coll.switch_orientation()
     new_positions = coll.get_positions()
-    assert_equal(props['orientation'], coll.get_orientation())
-    assert_equal(True, coll.is_horizontal())
+    assert props['orientation'] == coll.get_orientation()
+    assert coll.is_horizontal()
     np.testing.assert_array_equal(props['positions'], new_positions)
     check_segments(coll,
                    new_positions,
@@ -248,7 +245,7 @@ def test__EventCollection__switch_orientation_2x():
     splt.set_title('EventCollection: switch_orientation 2x')
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_orientation'])
+@image_comparison(['EventCollection_plot__set_orientation'])
 def test__EventCollection__set_orientation():
     '''
     check to make sure set_orientation works properly
@@ -256,8 +253,8 @@ def test__EventCollection__set_orientation():
     splt, coll, props = generate_EventCollection_plot()
     new_orientation = 'vertical'
     coll.set_orientation(new_orientation)
-    assert_equal(new_orientation, coll.get_orientation())
-    assert_equal(False, coll.is_horizontal())
+    assert new_orientation == coll.get_orientation()
+    assert not coll.is_horizontal()
     check_segments(coll,
                    props['positions'],
                    props['linelength'],
@@ -268,7 +265,7 @@ def test__EventCollection__set_orientation():
     splt.set_xlim(0, 2)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_linelength'])
+@image_comparison(['EventCollection_plot__set_linelength'])
 def test__EventCollection__set_linelength():
     '''
     check to make sure set_linelength works properly
@@ -276,7 +273,7 @@ def test__EventCollection__set_linelength():
     splt, coll, props = generate_EventCollection_plot()
     new_linelength = 15
     coll.set_linelength(new_linelength)
-    assert_equal(new_linelength, coll.get_linelength())
+    assert new_linelength == coll.get_linelength()
     check_segments(coll,
                    props['positions'],
                    new_linelength,
@@ -286,7 +283,7 @@ def test__EventCollection__set_linelength():
     splt.set_ylim(-20, 20)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_lineoffset'])
+@image_comparison(['EventCollection_plot__set_lineoffset'])
 def test__EventCollection__set_lineoffset():
     '''
     check to make sure set_lineoffset works properly
@@ -294,7 +291,7 @@ def test__EventCollection__set_lineoffset():
     splt, coll, props = generate_EventCollection_plot()
     new_lineoffset = -5.
     coll.set_lineoffset(new_lineoffset)
-    assert_equal(new_lineoffset, coll.get_lineoffset())
+    assert new_lineoffset == coll.get_lineoffset()
     check_segments(coll,
                    props['positions'],
                    props['linelength'],
@@ -304,7 +301,7 @@ def test__EventCollection__set_lineoffset():
     splt.set_ylim(-6, -4)
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_linestyle'])
+@image_comparison(['EventCollection_plot__set_linestyle'])
 def test__EventCollection__set_linestyle():
     '''
     check to make sure set_linestyle works properly
@@ -312,12 +309,11 @@ def test__EventCollection__set_linestyle():
     splt, coll, _ = generate_EventCollection_plot()
     new_linestyle = 'dashed'
     coll.set_linestyle(new_linestyle)
-    assert_equal(coll.get_linestyle(), [(0, (6.0, 6.0))])
+    assert coll.get_linestyle() == [(0, (6.0, 6.0))]
     splt.set_title('EventCollection: set_linestyle')
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_ls_dash'],
-                  remove_text=True)
+@image_comparison(['EventCollection_plot__set_ls_dash'], remove_text=True)
 def test__EventCollection__set_linestyle_single_dash():
     '''
     check to make sure set_linestyle accepts a single dash pattern
@@ -325,11 +321,11 @@ def test__EventCollection__set_linestyle_single_dash():
     splt, coll, _ = generate_EventCollection_plot()
     new_linestyle = (0, (6., 6.))
     coll.set_linestyle(new_linestyle)
-    assert_equal(coll.get_linestyle(), [(0, (6.0, 6.0))])
+    assert coll.get_linestyle() == [(0, (6.0, 6.0))]
     splt.set_title('EventCollection: set_linestyle')
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_linewidth'])
+@image_comparison(['EventCollection_plot__set_linewidth'])
 def test__EventCollection__set_linewidth():
     '''
     check to make sure set_linestyle works properly
@@ -337,11 +333,11 @@ def test__EventCollection__set_linewidth():
     splt, coll, _ = generate_EventCollection_plot()
     new_linewidth = 5
     coll.set_linewidth(new_linewidth)
-    assert_equal(coll.get_linewidth(), new_linewidth)
+    assert coll.get_linewidth() == new_linewidth
     splt.set_title('EventCollection: set_linewidth')
 
 
-@image_comparison(baseline_images=['EventCollection_plot__set_color'])
+@image_comparison(['EventCollection_plot__set_color'])
 def test__EventCollection__set_color():
     '''
     check to make sure set_color works properly
@@ -376,10 +372,10 @@ def check_segments(coll, positions, linelength, lineoffset, orientation):
 
     # test to make sure each segment is correct
     for i, segment in enumerate(segments):
-        assert_equal(segment[0, pos1], lineoffset + linelength / 2.)
-        assert_equal(segment[1, pos1], lineoffset - linelength / 2.)
-        assert_equal(segment[0, pos2], positions[i])
-        assert_equal(segment[1, pos2], positions[i])
+        assert segment[0, pos1] == lineoffset + linelength / 2
+        assert segment[1, pos1] == lineoffset - linelength / 2
+        assert segment[0, pos2] == positions[i]
+        assert segment[1, pos2] == positions[i]
 
 
 def check_allprop_array(values, target):
@@ -401,14 +397,14 @@ def test_null_collection_datalim():
 
 def test_add_collection():
     # Test if data limits are unchanged by adding an empty collection.
-    # Github issue #1490, pull #1497.
+    # GitHub issue #1490, pull #1497.
     plt.figure()
     ax = plt.axes()
     coll = ax.scatter([0, 1], [0, 1])
     ax.add_collection(coll)
     bounds = ax.dataLim.bounds
     coll = ax.scatter([], [])
-    assert_equal(ax.dataLim.bounds, bounds)
+    assert ax.dataLim.bounds == bounds
 
 
 def test_quiver_limits():
@@ -416,7 +412,7 @@ def test_quiver_limits():
     x, y = np.arange(8), np.arange(10)
     u = v = np.linspace(0, 10, 80).reshape(10, 8)
     q = plt.quiver(x, y, u, v)
-    assert_equal(q.get_datalim(ax.transData).bounds, (0., 0., 7., 9.))
+    assert q.get_datalim(ax.transData).bounds == (0., 0., 7., 9.)
 
     plt.figure()
     ax = plt.axes()
@@ -425,7 +421,7 @@ def test_quiver_limits():
     y, x = np.meshgrid(y, x)
     trans = mtransforms.Affine2D().translate(25, 32) + ax.transData
     plt.quiver(x, y, np.sin(x), np.cos(y), transform=trans)
-    assert_equal(ax.dataLim.bounds, (20.0, 30.0, 15.0, 6.0))
+    assert ax.dataLim.bounds == (20.0, 30.0, 15.0, 6.0)
 
 
 def test_barb_limits():
@@ -442,9 +438,8 @@ def test_barb_limits():
                               decimal=1)
 
 
-@image_comparison(baseline_images=['EllipseCollection_test_image'],
-                  extensions=['png'],
-                  remove_text=True)
+@image_comparison(['EllipseCollection_test_image.png'], remove_text=True,
+                  tol={'aarch64': 0.02}.get(platform.machine(), 0.0))
 def test_EllipseCollection():
     # Test basic functionality
     fig, ax = plt.subplots()
@@ -466,8 +461,7 @@ def test_EllipseCollection():
     ax.autoscale_view()
 
 
-@image_comparison(baseline_images=['polycollection_close'],
-                  extensions=['png'], remove_text=True)
+@image_comparison(['polycollection_close.png'], remove_text=True)
 def test_polycollection_close():
     from mpl_toolkits.mplot3d import Axes3D
 
@@ -504,8 +498,7 @@ def test_polycollection_close():
     ax.set_ylim3d(0, 4)
 
 
-@image_comparison(baseline_images=['regularpolycollection_rotate'],
-                  extensions=['png'], remove_text=True)
+@image_comparison(['regularpolycollection_rotate.png'], remove_text=True)
 def test_regularpolycollection_rotate():
     xx, yy = np.mgrid[:10, :10]
     xy_points = np.transpose([xx.flatten(), yy.flatten()])
@@ -520,15 +513,13 @@ def test_regularpolycollection_rotate():
     ax.autoscale_view()
 
 
-@image_comparison(baseline_images=['regularpolycollection_scale'],
-                  extensions=['png'], remove_text=True)
+@image_comparison(['regularpolycollection_scale.png'], remove_text=True)
 def test_regularpolycollection_scale():
     # See issue #3860
 
     class SquareCollection(mcollections.RegularPolyCollection):
         def __init__(self, **kwargs):
-            super(SquareCollection, self).__init__(
-                4, rotation=np.pi/4., **kwargs)
+            super().__init__(4, rotation=np.pi/4., **kwargs)
 
         def get_transform(self):
             """Return transform scaling circle areas to data space."""
@@ -557,7 +548,7 @@ def test_picking():
     col = ax.scatter([0], [0], [1000], picker=True)
     fig.savefig(io.BytesIO(), dpi=fig.dpi)
 
-    class MouseEvent(object):
+    class MouseEvent:
         pass
     event = MouseEvent()
     event.x = 325
@@ -573,8 +564,7 @@ def test_linestyle_single_dashes():
     plt.draw()
 
 
-@image_comparison(baseline_images=['size_in_xy'], remove_text=True,
-                  extensions=['png'])
+@image_comparison(['size_in_xy.png'], remove_text=True)
 def test_size_in_xy():
     fig, ax = plt.subplots()
 
@@ -616,32 +606,31 @@ def test_lslw_bcast():
     col.set_linestyles(['-', '-'])
     col.set_linewidths([1, 2, 3])
 
-    assert_equal(col.get_linestyles(), [(None, None)] * 6)
-    assert_equal(col.get_linewidths(), [1, 2, 3] * 2)
+    assert col.get_linestyles() == [(None, None)] * 6
+    assert col.get_linewidths() == [1, 2, 3] * 2
 
     col.set_linestyles(['-', '-', '-'])
-    assert_equal(col.get_linestyles(), [(None, None)] * 3)
-    assert_equal(col.get_linewidths(), [1, 2, 3])
+    assert col.get_linestyles() == [(None, None)] * 3
+    assert (col.get_linewidths() == [1, 2, 3]).all()
 
 
 @pytest.mark.style('default')
 def test_capstyle():
     col = mcollections.PathCollection([], capstyle='round')
-    assert_equal(col.get_capstyle(), 'round')
+    assert col.get_capstyle() == 'round'
     col.set_capstyle('butt')
-    assert_equal(col.get_capstyle(), 'butt')
+    assert col.get_capstyle() == 'butt'
 
 
 @pytest.mark.style('default')
 def test_joinstyle():
     col = mcollections.PathCollection([], joinstyle='round')
-    assert_equal(col.get_joinstyle(), 'round')
+    assert col.get_joinstyle() == 'round'
     col.set_joinstyle('miter')
-    assert_equal(col.get_joinstyle(), 'miter')
+    assert col.get_joinstyle() == 'miter'
 
 
-@image_comparison(baseline_images=['cap_and_joinstyle'],
-                  extensions=['png'])
+@image_comparison(['cap_and_joinstyle.png'])
 def test_cap_and_joinstyle_image():
     fig = plt.figure()
     ax = fig.add_subplot(1, 1, 1)
@@ -662,12 +651,91 @@ def test_cap_and_joinstyle_image():
     ax.set_title('Line collection with customized caps and joinstyle')
 
 
-@image_comparison(baseline_images=['scatter_post_alpha'],
-                  extensions=['png'], remove_text=True,
-                  style='default')
+@image_comparison(['scatter_post_alpha.png'],
+                  remove_text=True, style='default')
 def test_scatter_post_alpha():
     fig, ax = plt.subplots()
     sc = ax.scatter(range(5), range(5), c=range(5))
     # this needs to be here to update internal state
     fig.canvas.draw()
     sc.set_alpha(.1)
+
+
+def test_pathcollection_legend_elements():
+    np.random.seed(19680801)
+    x, y = np.random.rand(2, 10)
+    y = np.random.rand(10)
+    c = np.random.randint(0, 5, size=10)
+    s = np.random.randint(10, 300, size=10)
+
+    fig, ax = plt.subplots()
+    sc = ax.scatter(x, y, c=c, s=s, cmap="jet", marker="o", linewidths=0)
+
+    h, l = sc.legend_elements(fmt="{x:g}")
+    assert len(h) == 5
+    assert_array_equal(np.array(l).astype(float), np.arange(5))
+    colors = np.array([line.get_color() for line in h])
+    colors2 = sc.cmap(np.arange(5)/4)
+    assert_array_equal(colors, colors2)
+    l1 = ax.legend(h, l, loc=1)
+
+    h2, lab2 = sc.legend_elements(num=9)
+    assert len(h2) == 9
+    l2 = ax.legend(h2, lab2, loc=2)
+
+    h, l = sc.legend_elements(prop="sizes", alpha=0.5, color="red")
+    alpha = np.array([line.get_alpha() for line in h])
+    assert_array_equal(alpha, 0.5)
+    color = np.array([line.get_markerfacecolor() for line in h])
+    assert_array_equal(color, "red")
+    l3 = ax.legend(h, l, loc=4)
+
+    h, l = sc.legend_elements(prop="sizes", num=4, fmt="{x:.2f}",
+                              func=lambda x: 2*x)
+    actsizes = [line.get_markersize() for line in h]
+    labeledsizes = np.sqrt(np.array(l).astype(float)/2)
+    assert_array_almost_equal(actsizes, labeledsizes)
+    l4 = ax.legend(h, l, loc=3)
+
+    import matplotlib.ticker as mticker
+    loc = mticker.MaxNLocator(nbins=9, min_n_ticks=9-1,
+                              steps=[1, 2, 2.5, 3, 5, 6, 8, 10])
+    h5, lab5 = sc.legend_elements(num=loc)
+    assert len(h2) == len(h5)
+
+    levels = [-1, 0, 55.4, 260]
+    h6, lab6 = sc.legend_elements(num=levels, prop="sizes", fmt="{x:g}")
+    assert_array_equal(np.array(lab6).astype(float), levels[2:])
+
+    for l in [l1, l2, l3, l4]:
+        ax.add_artist(l)
+
+    fig.canvas.draw()
+
+
+def test_EventCollection_nosort():
+    # Check that EventCollection doesn't modify input in place
+    arr = np.array([3, 2, 1, 10])
+    coll = EventCollection(arr)
+    np.testing.assert_array_equal(arr, np.array([3, 2, 1, 10]))
+
+
+def test_blended_collection_autolim():
+    a = [1, 2, 4]
+    height = .2
+
+    xy_pairs = np.column_stack([np.repeat(a, 2), np.tile([0, height], len(a))])
+    line_segs = xy_pairs.reshape([len(a), 2, 2])
+
+    f, ax = plt.subplots()
+    trans = mtransforms.blended_transform_factory(ax.transData, ax.transAxes)
+    ax.add_collection(LineCollection(line_segs, transform=trans))
+    ax.autoscale_view(scalex=True, scaley=False)
+    np.testing.assert_allclose(ax.get_xlim(), [1., 4.])
+
+
+def test_singleton_autolim():
+    fig, ax = plt.subplots()
+    ax.scatter(0, 0)
+    np.testing.assert_allclose(ax.get_ylim(), [-0.06, 0.06])
+    np.testing.assert_allclose(ax.get_xlim(), [-0.06, 0.06])
