@@ -3,7 +3,7 @@ import matplotlib.pyplot as plt
 
 def test_stem_remove():
     ax = plt.gca()
-    st = ax.stem([1, 2], [1, 2])
+    st = ax.stem([1, 2], [1, 2], use_line_collection=True)
     st.remove()
 
 
