@@ -1,26 +1,20 @@
 import datetime
-import tempfile
-from unittest.mock import Mock
+try:
+    from contextlib import nullcontext
+except ImportError:
+    from contextlib import ExitStack as nullcontext  # Py 3.6.
 
 import dateutil.tz
 import dateutil.rrule
 import numpy as np
 import pytest
 
-from matplotlib.testing.decorators import image_comparison
-import matplotlib.pyplot as plt
+from matplotlib import rc_context
 from matplotlib.cbook import MatplotlibDeprecationWarning
 import matplotlib.dates as mdates
+import matplotlib.pyplot as plt
+from matplotlib.testing.decorators import image_comparison
 import matplotlib.ticker as mticker
-from matplotlib import rc_context
-
-
-def __has_pytz():
-    try:
-        import pytz
-        return True
-    except ImportError:
-        return False
 
 
 def test_date_numpyx():
@@ -33,12 +27,12 @@ def test_date_numpyx():
     ax = fig.add_subplot(1, 1, 1)
     h, = ax.plot(time, data)
     hnp, = ax.plot(timenp, data)
-    assert np.array_equal(h.get_xdata(orig=False), hnp.get_xdata(orig=False))
+    np.testing.assert_equal(h.get_xdata(orig=False), hnp.get_xdata(orig=False))
     fig = plt.figure(figsize=(10, 2))
     ax = fig.add_subplot(1, 1, 1)
     h, = ax.plot(data, time)
     hnp, = ax.plot(data, timenp)
-    assert np.array_equal(h.get_ydata(orig=False), hnp.get_ydata(orig=False))
+    np.testing.assert_equal(h.get_ydata(orig=False), hnp.get_ydata(orig=False))
 
 
 @pytest.mark.parametrize('t0', [datetime.datetime(2017, 1, 1, 0, 1, 1),
@@ -58,7 +52,7 @@ def test_date_date2num_numpy(t0, dtype):
     time = mdates.date2num(t0)
     tnp = np.array(t0, dtype=dtype)
     nptime = mdates.date2num(tnp)
-    assert np.array_equal(time, nptime)
+    np.testing.assert_equal(time, nptime)
 
 
 @pytest.mark.parametrize('dtype', ['datetime64[s]',
@@ -79,9 +73,9 @@ def test_date2num_NaT_scalar(units):
     assert np.isnan(tmpl)
 
 
-@image_comparison(baseline_images=['date_empty'], extensions=['png'])
+@image_comparison(['date_empty.png'])
 def test_date_empty():
-    # make sure mpl does the right thing when told to plot dates even
+    # make sure we do the right thing when told to plot dates even
     # if no date data has been presented, cf
     # http://sourceforge.net/tracker/?func=detail&aid=2850075&group_id=80706&atid=560720
     fig = plt.figure()
@@ -89,7 +83,7 @@ def test_date_empty():
     ax.xaxis_date()
 
 
-@image_comparison(baseline_images=['date_axhspan'], extensions=['png'])
+@image_comparison(['date_axhspan.png'])
 def test_date_axhspan():
     # test ax hspan with date inputs
     t0 = datetime.datetime(2009, 1, 20)
@@ -102,7 +96,7 @@ def test_date_axhspan():
     fig.subplots_adjust(left=0.25)
 
 
-@image_comparison(baseline_images=['date_axvspan'], extensions=['png'])
+@image_comparison(['date_axvspan.png'])
 def test_date_axvspan():
     # test ax hspan with date inputs
     t0 = datetime.datetime(2000, 1, 20)
@@ -115,8 +109,7 @@ def test_date_axvspan():
     fig.autofmt_xdate()
 
 
-@image_comparison(baseline_images=['date_axhline'],
-                  extensions=['png'])
+@image_comparison(['date_axhline.png'])
 def test_date_axhline():
     # test ax hline with date inputs
     t0 = datetime.datetime(2009, 1, 20)
@@ -129,8 +122,7 @@ def test_date_axhline():
     fig.subplots_adjust(left=0.25)
 
 
-@image_comparison(baseline_images=['date_axvline'],
-                  extensions=['png'])
+@image_comparison(['date_axvline.png'])
 def test_date_axvline():
     # test ax hline with date inputs
     t0 = datetime.datetime(2000, 1, 20)
@@ -143,12 +135,13 @@ def test_date_axvline():
     fig.autofmt_xdate()
 
 
-def test_too_many_date_ticks():
+def test_too_many_date_ticks(caplog):
     # Attempt to test SF 2715172, see
     # https://sourceforge.net/tracker/?func=detail&aid=2715172&group_id=80706&atid=560720
     # setting equal datetimes triggers and expander call in
     # transforms.nonsingular which results in too many ticks in the
-    # DayLocator.  This should trigger a Locator.MAXTICKS RuntimeError
+    # DayLocator.  This should emit a log at WARNING level.
+    caplog.set_level("WARNING")
     t0 = datetime.datetime(2000, 1, 20)
     tf = datetime.datetime(2000, 1, 20)
     fig = plt.figure()
@@ -160,11 +153,16 @@ def test_too_many_date_ticks():
             'Attempting to set identical left == right' in str(rec[0].message)
     ax.plot([], [])
     ax.xaxis.set_major_locator(mdates.DayLocator())
-    with pytest.raises(RuntimeError):
-        fig.savefig('junk.png')
+    fig.canvas.draw()
+    # The warning is emitted multiple times because the major locator is also
+    # called both when placing the minor ticks (for overstriking detection) and
+    # during tick label positioning.
+    assert caplog.records and all(
+        record.name == "matplotlib.ticker" and record.levelname == "WARNING"
+        for record in caplog.records)
 
 
-@image_comparison(baseline_images=['RRuleLocator_bounds'], extensions=['png'])
+@image_comparison(['RRuleLocator_bounds.png'])
 def test_RRuleLocator():
     import matplotlib.testing.jpl_units as units
     units.register()
@@ -197,8 +195,7 @@ def test_RRuleLocator_dayrange():
     # On success, no overflow error shall be thrown
 
 
-@image_comparison(baseline_images=['DateFormatter_fractionalSeconds'],
-                  extensions=['png'])
+@image_comparison(['DateFormatter_fractionalSeconds.png'])
 def test_DateFormatter():
     import matplotlib.testing.jpl_units as units
     units.register()
@@ -255,62 +252,15 @@ def test_locator_set_formatter():
     assert ticklabels == expected
 
 
-def test_date_formatter_strftime():
-    """
-    Tests that DateFormatter matches datetime.strftime,
-    check microseconds for years before 1900 for bug #3179
-    as well as a few related issues for years before 1900.
-    """
-    def test_strftime_fields(dt):
-        """For datetime object dt, check DateFormatter fields"""
-        # Note: the last couple of %%s are to check multiple %s are handled
-        # properly; %% should get replaced by %.
-        formatter = mdates.DateFormatter("%w %d %m %y %Y %H %I %M %S %%%f %%x")
-        # Compute date fields without using datetime.strftime,
-        # since datetime.strftime does not work before year 1900
-        formatted_date_str = (
-            "{weekday} {day:02d} {month:02d} {year:02d} {full_year:04d} "
-            "{hour24:02d} {hour12:02d} {minute:02d} {second:02d} "
-            "%{microsecond:06d} %x"
-            .format(
-                weekday=str((dt.weekday() + 1) % 7),
-                day=dt.day,
-                month=dt.month,
-                year=dt.year % 100,
-                full_year=dt.year,
-                hour24=dt.hour,
-                hour12=((dt.hour-1) % 12) + 1,
-                minute=dt.minute,
-                second=dt.second,
-                microsecond=dt.microsecond))
-        with pytest.warns(MatplotlibDeprecationWarning):
-            assert formatter.strftime(dt) == formatted_date_str
-
-        try:
-            # Test strftime("%x") with the current locale.
-            import locale  # Might not exist on some platforms, such as Windows
-            locale_formatter = mdates.DateFormatter("%x")
-            locale_d_fmt = locale.nl_langinfo(locale.D_FMT)
-            expanded_formatter = mdates.DateFormatter(locale_d_fmt)
-            with pytest.warns(MatplotlibDeprecationWarning):
-                assert locale_formatter.strftime(dt) == \
-                    expanded_formatter.strftime(dt)
-        except (ImportError, AttributeError):
-            pass
-
-    for year in range(1, 3000, 71):
-        # Iterate through random set of years
-        test_strftime_fields(datetime.datetime(year, 1, 1))
-        test_strftime_fields(datetime.datetime(year, 2, 3, 4, 5, 6, 12345))
-
-
 def test_date_formatter_callable():
-    scale = -11
-    locator = Mock(_get_unit=Mock(return_value=scale))
-    callable_formatting_function = (lambda dates, _:
-                                    [dt.strftime('%d-%m//%Y') for dt in dates])
 
-    formatter = mdates.AutoDateFormatter(locator)
+    class _Locator:
+        def _get_unit(self): return -11
+
+    def callable_formatting_function(dates, _):
+        return [dt.strftime('%d-%m//%Y') for dt in dates]
+
+    formatter = mdates.AutoDateFormatter(_Locator())
     formatter.scaled[-10] = callable_formatting_function
     assert formatter([datetime.datetime(2014, 12, 25)]) == ['25-12//2014']
 
@@ -358,9 +308,8 @@ def test_empty_date_with_year_formatter():
     yearFmt = dates.DateFormatter('%Y')
     ax.xaxis.set_major_formatter(yearFmt)
 
-    with tempfile.TemporaryFile() as fh:
-        with pytest.raises(ValueError):
-            fig.savefig(fh)
+    with pytest.raises(ValueError):
+        fig.canvas.draw()
 
 
 def test_auto_date_locator():
@@ -430,7 +379,9 @@ def test_auto_date_locator():
     for t_delta, expected in results:
         d2 = d1 + t_delta
         locator = _create_auto_date_locator(d1, d2)
-        assert list(map(str, mdates.num2date(locator()))) == expected
+        with (pytest.warns(UserWarning) if t_delta.microseconds
+              else nullcontext()):
+            assert list(map(str, mdates.num2date(locator()))) == expected
 
 
 def test_auto_date_locator_intmult():
@@ -505,7 +456,9 @@ def test_auto_date_locator_intmult():
     for t_delta, expected in results:
         d2 = d1 + t_delta
         locator = _create_auto_date_locator(d1, d2)
-        assert list(map(str, mdates.num2date(locator()))) == expected
+        with (pytest.warns(UserWarning) if t_delta.microseconds
+              else nullcontext()):
+            assert list(map(str, mdates.num2date(locator()))) == expected
 
 
 def test_concise_formatter():
@@ -559,6 +512,49 @@ def test_concise_formatter():
         assert strings == expected
 
 
+def test_concise_formatter_tz():
+    def _create_auto_date_locator(date1, date2, tz):
+        fig, ax = plt.subplots()
+
+        locator = mdates.AutoDateLocator(interval_multiples=True)
+        formatter = mdates.ConciseDateFormatter(locator, tz=tz)
+        ax.yaxis.set_major_locator(locator)
+        ax.yaxis.set_major_formatter(formatter)
+        ax.set_ylim(date1, date2)
+        fig.canvas.draw()
+        sts = []
+        for st in ax.get_yticklabels():
+            sts += [st.get_text()]
+        return sts, ax.yaxis.get_offset_text().get_text()
+
+    d1 = datetime.datetime(1997, 1, 1).replace(tzinfo=datetime.timezone.utc)
+    results = ([datetime.timedelta(hours=40),
+                ['03:00', '07:00', '11:00', '15:00', '19:00', '23:00',
+                 '03:00', '07:00', '11:00', '15:00', '19:00'],
+                "1997-Jan-02"
+                ],
+               [datetime.timedelta(minutes=20),
+                ['03:00', '03:05', '03:10', '03:15', '03:20'],
+                "1997-Jan-01"
+                ],
+               [datetime.timedelta(seconds=40),
+                ['03:00', '05', '10', '15', '20', '25', '30', '35', '40'],
+                "1997-Jan-01 03:00"
+                ],
+               [datetime.timedelta(seconds=2),
+                ['59.5', '03:00', '00.5', '01.0', '01.5', '02.0', '02.5'],
+                "1997-Jan-01 03:00"
+                ],
+               )
+
+    new_tz = datetime.timezone(datetime.timedelta(hours=3))
+    for t_delta, expected_strings, expected_offset in results:
+        d2 = d1 + t_delta
+        strings, offset = _create_auto_date_locator(d1, d2, new_tz)
+        assert strings == expected_strings
+        assert offset == expected_offset
+
+
 def test_auto_date_locator_intmult_tz():
     def _create_auto_date_locator(date1, date2, tz):
         locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
@@ -630,8 +626,7 @@ def test_auto_date_locator_intmult_tz():
             assert st == expected
 
 
-@image_comparison(baseline_images=['date_inverted_limit'],
-                  extensions=['png'])
+@image_comparison(['date_inverted_limit.png'])
 def test_date_inverted_limit():
     # test ax hline with date inputs
     t0 = datetime.datetime(2009, 1, 20)
@@ -763,10 +758,9 @@ def test_rrulewrapper():
 
 
 @pytest.mark.pytz
-@pytest.mark.skipif(not __has_pytz(), reason="Requires pytz")
 def test_rrulewrapper_pytz():
     # Test to make sure pytz zones are supported in rrules
-    import pytz
+    pytz = pytest.importorskip("pytz")
 
     def attach_tz(dt, zi):
         return zi.localize(dt)
@@ -775,9 +769,8 @@ def test_rrulewrapper_pytz():
 
 
 @pytest.mark.pytz
-@pytest.mark.skipif(not __has_pytz(), reason="Requires pytz")
 def test_yearlocator_pytz():
-    import pytz
+    pytz = pytest.importorskip("pytz")
 
     tz = pytz.timezone('America/New_York')
     x = [tz.localize(datetime.datetime(2010, 1, 1))
@@ -827,4 +820,4 @@ def test_num2timedelta(x, tdelta):
 def test_datetime64_in_list():
     dt = [np.datetime64('2000-01-01'), np.datetime64('2001-01-01')]
     dn = mdates.date2num(dt)
-    assert np.array_equal(dn, [730120.,  730486.])
+    np.testing.assert_equal(dn, [730120.,  730486.])
