@@ -1,67 +1,62 @@
-from __future__ import absolute_import, division, print_function
-
-import six
-
-import os
-import warnings
 from collections import OrderedDict
+import copy
+import os
+from pathlib import Path
+import subprocess
+import sys
+from unittest import mock
 
 from cycler import cycler, Cycler
 import pytest
 
-try:
-    from unittest import mock
-except ImportError:
-    import mock
 import matplotlib as mpl
+from matplotlib import cbook
 import matplotlib.pyplot as plt
 import matplotlib.colors as mcolors
-from itertools import chain
 import numpy as np
 from matplotlib.rcsetup import (validate_bool_maybe_none,
                                 validate_stringlist,
                                 validate_colorlist,
                                 validate_color,
                                 validate_bool,
+                                validate_fontweight,
                                 validate_nseq_int,
                                 validate_nseq_float,
                                 validate_cycler,
                                 validate_hatch,
                                 validate_hist_bins,
+                                validate_markevery,
                                 _validate_linestyle)
 
 
-mpl.rc('text', usetex=False)
-mpl.rc('lines', linewidth=22)
+def test_rcparams(tmpdir):
+    mpl.rc('text', usetex=False)
+    mpl.rc('lines', linewidth=22)
 
-fname = os.path.join(os.path.dirname(__file__), 'test_rcparams.rc')
-
-
-def test_rcparams():
     usetex = mpl.rcParams['text.usetex']
     linewidth = mpl.rcParams['lines.linewidth']
 
+    rcpath = Path(tmpdir) / 'test_rcparams.rc'
+    rcpath.write_text('lines.linewidth: 33')
+
     # test context given dictionary
     with mpl.rc_context(rc={'text.usetex': not usetex}):
         assert mpl.rcParams['text.usetex'] == (not usetex)
     assert mpl.rcParams['text.usetex'] == usetex
 
-    # test context given filename (mpl.rc sets linewdith to 33)
-    with mpl.rc_context(fname=fname):
+    # test context given filename (mpl.rc sets linewidth to 33)
+    with mpl.rc_context(fname=rcpath):
         assert mpl.rcParams['lines.linewidth'] == 33
     assert mpl.rcParams['lines.linewidth'] == linewidth
 
     # test context given filename and dictionary
-    with mpl.rc_context(fname=fname, rc={'lines.linewidth': 44}):
+    with mpl.rc_context(fname=rcpath, rc={'lines.linewidth': 44}):
         assert mpl.rcParams['lines.linewidth'] == 44
     assert mpl.rcParams['lines.linewidth'] == linewidth
 
     # test rc_file
-    try:
-        mpl.rc_file(fname)
-        assert mpl.rcParams['lines.linewidth'] == 33
-    finally:
-        mpl.rcParams['lines.linewidth'] = linewidth
+    mpl.rc_file(rcpath)
+    assert mpl.rcParams['lines.linewidth'] == 33
 
 
 def test_RcParams_class():
@@ -101,22 +96,15 @@ def test_rcparams_update():
     rc = mpl.RcParams({'figure.figsize': (3.5, 42)})
     bad_dict = {'figure.figsize': (3.5, 42, 1)}
     # make sure validation happens on input
-    with pytest.raises(ValueError):
-
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore',
-                                message='.*(validate)',
-                                category=UserWarning)
-            rc.update(bad_dict)
+    with pytest.raises(ValueError), \
+         pytest.warns(UserWarning, match="validate"):
+        rc.update(bad_dict)
 
 
 def test_rcparams_init():
-    with pytest.raises(ValueError):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore',
-                                message='.*(validate)',
-                                category=UserWarning)
-            mpl.RcParams({'figure.figsize': (3.5, 42, 1)})
+    with pytest.raises(ValueError), \
+         pytest.warns(UserWarning, match="validate"):
+        mpl.RcParams({'figure.figsize': (3.5, 42, 1)})
 
 
 def test_Bug_2543():
@@ -127,18 +115,13 @@ def test_Bug_2543():
     # We filter warnings at this stage since a number of them are raised
     # for deprecated rcparams as they should. We don't want these in the
     # printed in the test suite.
-    with warnings.catch_warnings():
-        warnings.filterwarnings('ignore',
-                                message='.*(deprecated|obsolete)',
-                                category=UserWarning)
+    with cbook._suppress_matplotlib_deprecation_warning():
         with mpl.rc_context():
             _copy = mpl.rcParams.copy()
             for key in _copy:
                 mpl.rcParams[key] = _copy[key]
-            mpl.rcParams['text.dvipnghack'] = None
         with mpl.rc_context():
-            from copy import deepcopy
-            _deep_copy = deepcopy(mpl.rcParams)
+            copy.deepcopy(mpl.rcParams)
         # real test is that this does not raise
         assert validate_bool_maybe_none(None) is None
         assert validate_bool_maybe_none("none") is None
@@ -175,8 +158,8 @@ legend_color_test_ids = [
 @pytest.mark.parametrize('color_type, param_dict, target', legend_color_tests,
                          ids=legend_color_test_ids)
 def test_legend_colors(color_type, param_dict, target):
-    param_dict['legend.%scolor' % (color_type, )] = param_dict.pop('color')
-    get_func = 'get_%scolor' % (color_type, )
+    param_dict[f'legend.{color_type}color'] = param_dict.pop('color')
+    get_func = f'get_{color_type}color'
 
     with mpl.rc_context(param_dict):
         _, ax = plt.subplots()
@@ -185,24 +168,43 @@ def test_legend_colors(color_type, param_dict, target):
         assert getattr(leg.legendPatch, get_func)() == target
 
 
-def test_Issue_1713():
-    utf32_be = os.path.join(os.path.dirname(__file__),
-                           'test_utf32_be_rcparams.rc')
-    import locale
+def test_mfc_rcparams():
+    mpl.rcParams['lines.markerfacecolor'] = 'r'
+    ln = mpl.lines.Line2D([1, 2], [1, 2])
+    assert ln.get_markerfacecolor() == 'r'
+
+
+def test_mec_rcparams():
+    mpl.rcParams['lines.markeredgecolor'] = 'r'
+    ln = mpl.lines.Line2D([1, 2], [1, 2])
+    assert ln.get_markeredgecolor() == 'r'
+
+
+def test_axes_titlecolor_rcparams():
+    mpl.rcParams['axes.titlecolor'] = 'r'
+    _, ax = plt.subplots()
+    title = ax.set_title("Title")
+    assert title.get_color() == 'r'
+
+
+def test_Issue_1713(tmpdir):
+    rcpath = Path(tmpdir) / 'test_rcparams.rc'
+    rcpath.write_text('timezone: UTC', encoding='UTF-32-BE')
     with mock.patch('locale.getpreferredencoding', return_value='UTF-32-BE'):
-        rc = mpl.rc_params_from_file(utf32_be, True, False)
+        rc = mpl.rc_params_from_file(rcpath, True, False)
     assert rc.get('timezone') == 'UTC'
 
 
 def generate_validator_testcases(valid):
     validation_tests = (
         {'validator': validate_bool,
-         'success': chain(((_, True) for _ in
-                           ('t', 'y', 'yes', 'on', 'true', '1', 1, True)),
-                           ((_, False) for _ in
-                            ('f', 'n', 'no', 'off', 'false', '0', 0, False))),
-        'fail': ((_, ValueError)
-                 for _ in ('aardvark', 2, -1, [], ))},
+         'success': (*((_, True) for _ in
+                       ('t', 'y', 'yes', 'on', 'true', '1', 1, True)),
+                     *((_, False) for _ in
+                       ('f', 'n', 'no', 'off', 'false', '0', 0, False))),
+         'fail': ((_, ValueError)
+                  for _ in ('aardvark', 2, -1, [], ))
+         },
         {'validator': validate_stringlist,
          'success': (('', []),
                      ('a,b', ['a', 'b']),
@@ -215,11 +217,11 @@ def generate_validator_testcases(valid):
                      (np.array(['a', 'b']), ['a', 'b']),
                      ((1, 2), ['1', '2']),
                      (np.array([1, 2]), ['1', '2']),
-                    ),
+                     ),
          'fail': ((dict(), ValueError),
                   (1, ValueError),
-                 )
-        },
+                  )
+         },
         {'validator': validate_nseq_int(2),
          'success': ((_, [1, 2])
                      for _ in ('1, 2', [1.5, 2.5], [1, 2],
@@ -228,7 +230,7 @@ def generate_validator_testcases(valid):
                   for _ in ('aardvark', ('a', 1),
                             (1, 2, 3)
                             ))
-        },
+         },
         {'validator': validate_nseq_float(2),
          'success': ((_, [1.5, 2.5])
                      for _ in ('1.5, 2.5', [1.5, 2.5], [1.5, 2.5],
@@ -237,7 +239,7 @@ def generate_validator_testcases(valid):
                   for _ in ('aardvark', ('a', 1),
                             (1, 2, 3)
                             ))
-        },
+         },
         {'validator': validate_cycler,
          'success': (('cycler("color", "rgb")',
                       cycler("color", 'rgb')),
@@ -246,17 +248,17 @@ def generate_validator_testcases(valid):
                      ("""(cycler("color", ["r", "g", "b"]) +
                           cycler("mew", [2, 3, 5]))""",
                       (cycler("color", 'rgb') +
-                          cycler("markeredgewidth", [2, 3, 5]))),
+                       cycler("markeredgewidth", [2, 3, 5]))),
                      ("cycler(c='rgb', lw=[1, 2, 3])",
                       cycler('color', 'rgb') + cycler('linewidth', [1, 2, 3])),
                      ("cycler('c', 'rgb') * cycler('linestyle', ['-', '--'])",
                       (cycler('color', 'rgb') *
-                          cycler('linestyle', ['-', '--']))),
+                       cycler('linestyle', ['-', '--']))),
                      (cycler('ls', ['-', '--']),
                       cycler('linestyle', ['-', '--'])),
                      (cycler(mew=[2, 5]),
                       cycler('markeredgewidth', [2, 5])),
-                    ),
+                     ),
          # This is *so* incredibly important: validate_cycler() eval's
          # an arbitrary string! I think I have it locked down enough,
          # and that is what this is testing.
@@ -267,63 +269,62 @@ def generate_validator_testcases(valid):
          'fail': ((4, ValueError),  # Gotta be a string or Cycler object
                   ('cycler("bleh, [])', ValueError),  # syntax error
                   ('Cycler("linewidth", [1, 2, 3])',
-                      ValueError),  # only 'cycler()' function is allowed
+                   ValueError),  # only 'cycler()' function is allowed
                   ('1 + 2', ValueError),  # doesn't produce a Cycler object
                   ('os.system("echo Gotcha")', ValueError),  # os not available
                   ('import os', ValueError),  # should not be able to import
                   ('def badjuju(a): return a; badjuju(cycler("color", "rgb"))',
-                      ValueError),  # Should not be able to define anything
-                                    # even if it does return a cycler
+                   ValueError),  # Should not be able to define anything
+                  # even if it does return a cycler
                   ('cycler("waka", [1, 2, 3])', ValueError),  # not a property
                   ('cycler(c=[1, 2, 3])', ValueError),  # invalid values
                   ("cycler(lw=['a', 'b', 'c'])", ValueError),  # invalid values
                   (cycler('waka', [1, 3, 5]), ValueError),  # not a property
                   (cycler('color', ['C1', 'r', 'g']), ValueError)  # no CN
-                 )
-        },
+                  )
+         },
         {'validator': validate_hatch,
          'success': (('--|', '--|'), ('\\oO', '\\oO'),
                      ('/+*/.x', '/+*/.x'), ('', '')),
          'fail': (('--_', ValueError),
-                  (8, ValueError),
-                  ('X', ValueError)),
-        },
+                 (8, ValueError),
+                 ('X', ValueError)),
+         },
         {'validator': validate_colorlist,
          'success': (('r,g,b', ['r', 'g', 'b']),
                      (['r', 'g', 'b'], ['r', 'g', 'b']),
                      ('r, ,', ['r']),
                      (['', 'g', 'blue'], ['g', 'blue']),
                      ([np.array([1, 0, 0]), np.array([0, 1, 0])],
-                         np.array([[1, 0, 0], [0, 1, 0]])),
+                     np.array([[1, 0, 0], [0, 1, 0]])),
                      (np.array([[1, 0, 0], [0, 1, 0]]),
-                         np.array([[1, 0, 0], [0, 1, 0]])),
-                    ),
+                     np.array([[1, 0, 0], [0, 1, 0]])),
+                     ),
          'fail': (('fish', ValueError),
-                 ),
-        },
+                  ),
+         },
         {'validator': validate_color,
          'success': (('None', 'none'),
                      ('none', 'none'),
                      ('AABBCC', '#AABBCC'),  # RGB hex code
                      ('AABBCC00', '#AABBCC00'),  # RGBA hex code
                      ('tab:blue', 'tab:blue'),  # named color
-                     ('C0', 'C0'),  # color from cycle
+                     ('C12', 'C12'),  # color from cycle
                      ('(0, 1, 0)', [0.0, 1.0, 0.0]),  # RGB tuple
                      ((0, 1, 0), (0, 1, 0)),  # non-string version
                      ('(0, 1, 0, 1)', [0.0, 1.0, 0.0, 1.0]),  # RGBA tuple
                      ((0, 1, 0, 1), (0, 1, 0, 1)),  # non-string version
                      ('(0, 1, "0.5")', [0.0, 1.0, 0.5]),  # unusual but valid
-
-                    ),
+                     ),
          'fail': (('tab:veryblue', ValueError),  # invalid name
-                  ('C123', ValueError),  # invalid RGB(A) code and cycle index
                   ('(0, 1)', ValueError),  # tuple with length < 3
                   ('(0, 1, 0, 1, 0)', ValueError),  # tuple with length > 4
                   ('(0, 1, none)', ValueError),  # cannot cast none to float
-                 ),
-        },
+                  ),
+         },
         {'validator': validate_hist_bins,
          'success': (('auto', 'auto'),
+                     ('fd', 'fd'),
                      ('10', 10),
                      ('1, 2, 3', [1, 2, 3]),
                      ([1, 2, 3], [1, 2, 3]),
@@ -331,45 +332,60 @@ def generate_validator_testcases(valid):
                      ),
          'fail': (('aardvark', ValueError),
                   )
-         }
+         },
+        {'validator': validate_markevery,
+         'success': ((None, None),
+                     (1, 1),
+                     (0.1, 0.1),
+                     ((1, 1), (1, 1)),
+                     ((0.1, 0.1), (0.1, 0.1)),
+                     ([1, 2, 3], [1, 2, 3]),
+                     (slice(2), slice(None, 2, None)),
+                     (slice(1, 2, 3), slice(1, 2, 3))
+                     ),
+         'fail': (((1, 2, 3), TypeError),
+                  ([1, 2, 0.3], TypeError),
+                  (['a', 2, 3], TypeError),
+                  ([1, 2, 'a'], TypeError),
+                  ((0.1, 0.2, 0.3), TypeError),
+                  ((0.1, 2, 3), TypeError),
+                  ((1, 0.2, 0.3), TypeError),
+                  ((1, 0.1), TypeError),
+                  ((0.1, 1), TypeError),
+                  (('abc'), TypeError),
+                  ((1, 'a'), TypeError),
+                  ((0.1, 'b'), TypeError),
+                  (('a', 1), TypeError),
+                  (('a', 0.1), TypeError),
+                  ('abc', TypeError),
+                  ('a', TypeError),
+                  (object(), TypeError)
+                  )
+         },
+        {'validator': _validate_linestyle,
+         'success': (('-', '-'), ('solid', 'solid'),
+                     ('--', '--'), ('dashed', 'dashed'),
+                     ('-.', '-.'), ('dashdot', 'dashdot'),
+                     (':', ':'), ('dotted', 'dotted'),
+                     ('', ''), (' ', ' '),
+                     ('None', 'none'), ('none', 'none'),
+                     ('DoTtEd', 'dotted'),  # case-insensitive
+                     (['1.23', '4.56'], (None, [1.23, 4.56])),
+                     ([1.23, 456], (None, [1.23, 456.0])),
+                     ([1, 2, 3, 4], (None, [1.0, 2.0, 3.0, 4.0])),
+                     ),
+         'fail': (('aardvark', ValueError),  # not a valid string
+                  (b'dotted', ValueError),
+                  ('dotted'.encode('utf-16'), ValueError),
+                  ((None, [1, 2]), ValueError),  # (offset, dashes) != OK
+                  ((0, [1, 2]), ValueError),  # idem
+                  ((-1, [1, 2]), ValueError),  # idem
+                  ([1, 2, 3], ValueError),  # sequence with odd length
+                  (1.23, ValueError),  # not a sequence
+                  )
+         },
     )
 
-    # The behavior of _validate_linestyle depends on the version of Python.
-    # ASCII-compliant bytes arguments should pass on Python 2 because of the
-    # automatic conversion between bytes and strings. Python 3 does not
-    # perform such a conversion, so the same cases should raise an exception.
-    #
-    # Common cases:
-    ls_test = {'validator': _validate_linestyle,
-               'success': (('-', '-'), ('solid', 'solid'),
-                           ('--', '--'), ('dashed', 'dashed'),
-                           ('-.', '-.'), ('dashdot', 'dashdot'),
-                           (':', ':'), ('dotted', 'dotted'),
-                           ('', ''), (' ', ' '),
-                           ('None', 'none'), ('none', 'none'),
-                           ('DoTtEd', 'dotted'),  # case-insensitive
-                           (['1.23', '4.56'], (None, [1.23, 4.56])),
-                           ([1.23, 456], (None, [1.23, 456.0])),
-                           ([1, 2, 3, 4], (None, [1.0, 2.0, 3.0, 4.0])),
-                          ),
-               'fail': (('aardvark', ValueError),  # not a valid string
-                        ('dotted'.encode('utf-16'), ValueError),  # even on PY2
-                        ((None, [1, 2]), ValueError),  # (offset, dashes) != OK
-                        ((0, [1, 2]), ValueError),  # idem
-                        ((-1, [1, 2]), ValueError),  # idem
-                        ([1, 2, 3], ValueError),  # sequence with odd length
-                        (1.23, ValueError),  # not a sequence
-                       )
-                }
-    # Add some cases of bytes arguments that Python 2 can convert silently:
-    ls_bytes_args = (b'dotted', 'dotted'.encode('ascii'))
-    if six.PY3:
-        ls_test['fail'] += tuple((arg, ValueError) for arg in ls_bytes_args)
-    else:
-        ls_test['success'] += tuple((arg, 'dotted') for arg in ls_bytes_args)
-    # Update the validation test sequence.
-    validation_tests += (ls_test,)
-
     for validator_dict in validation_tests:
         validator = validator_dict['validator']
         if valid:
@@ -385,7 +401,7 @@ def generate_validator_testcases(valid):
 def test_validator_valid(validator, arg, target):
     res = validator(arg)
     if isinstance(target, np.ndarray):
-        assert np.all(res == target)
+        np.testing.assert_equal(res, target)
     elif not isinstance(target, Cycler):
         assert res == target
     else:
@@ -400,6 +416,26 @@ def test_validator_invalid(validator, arg, exception_type):
         validator(arg)
 
 
+@pytest.mark.parametrize('weight, parsed_weight', [
+    ('bold', 'bold'),
+    ('BOLD', ValueError),  # weight is case-sensitive
+    (100, 100),
+    ('100', 100),
+    (np.array(100), 100),
+    # fractional fontweights are not defined. This should actually raise a
+    # ValueError, but historically did not.
+    (20.6, 20),
+    ('20.6', ValueError),
+    ([100], ValueError),
+])
+def test_validate_fontweight(weight, parsed_weight):
+    if parsed_weight is ValueError:
+        with pytest.raises(ValueError):
+            validate_fontweight(weight)
+    else:
+        assert validate_fontweight(weight) == parsed_weight
+
+
 def test_keymaps():
     key_list = [k for k in mpl.rcParams if 'keymap' in k]
     for k in key_list:
@@ -425,22 +461,16 @@ def test_rcparams_reset_after_fail():
 
 
 def test_if_rctemplate_is_up_to_date():
-    # This tests if the matplotlibrc.template file
-    # contains all valid rcParams.
-    dep1 = mpl._all_deprecated
-    dep2 = mpl._deprecated_set
-    deprecated = list(dep1.union(dep2))
-    path_to_rc = os.path.join(mpl.get_data_path(), 'matplotlibrc')
-    with open(path_to_rc, "r") as f:
-        rclines = f.readlines()
+    # This tests if the matplotlibrc.template file contains all valid rcParams.
+    deprecated = {*mpl._all_deprecated, *mpl._deprecated_remain_as_none}
+    with cbook._get_data_path('matplotlibrc').open() as file:
+        rclines = file.readlines()
     missing = {}
     for k, v in mpl.defaultParams.items():
         if k[0] == "_":
             continue
         if k in deprecated:
             continue
-        if "verbose" in k:
-            continue
         found = False
         for line in rclines:
             if k in line:
@@ -448,17 +478,16 @@ def test_if_rctemplate_is_up_to_date():
         if not found:
             missing.update({k: v})
     if missing:
-        raise ValueError("The following params are missing " +
-                         "in the matplotlibrc.template file: {}"
+        raise ValueError("The following params are missing in the "
+                         "matplotlibrc.template file: {}"
                          .format(missing.items()))
 
 
 def test_if_rctemplate_would_be_valid(tmpdir):
     # This tests if the matplotlibrc.template file would result in a valid
     # rc file if all lines are uncommented.
-    path_to_rc = os.path.join(mpl.get_data_path(), 'matplotlibrc')
-    with open(path_to_rc, "r") as f:
-        rclines = f.readlines()
+    with cbook._get_data_path('matplotlibrc').open() as file:
+        rclines = file.readlines()
     newlines = []
     for line in rclines:
         if line[0] == "#":
@@ -479,3 +508,28 @@ def test_if_rctemplate_would_be_valid(tmpdir):
                                 fail_on_error=True,
                                 use_default_template=False)
         assert len(record) == 0
+
+
+@pytest.mark.skipif(sys.platform != "linux", reason="Linux only")
+def test_backend_fallback_headless(tmpdir):
+    env = {**os.environ,
+           "DISPLAY": "", "MPLBACKEND": "", "MPLCONFIGDIR": str(tmpdir)}
+    with pytest.raises(subprocess.CalledProcessError):
+        subprocess.run(
+            [sys.executable, "-c",
+             "import matplotlib; matplotlib.use('tkagg')"],
+            env=env, check=True)
+
+
+@pytest.mark.skipif(sys.platform == "linux" and not os.environ.get("DISPLAY"),
+                    reason="headless")
+def test_backend_fallback_headful(tmpdir):
+    pytest.importorskip("tkinter")
+    env = {**os.environ, "MPLBACKEND": "", "MPLCONFIGDIR": str(tmpdir)}
+    backend = subprocess.check_output(
+        [sys.executable, "-c",
+         "import matplotlib.pyplot; print(matplotlib.get_backend())"],
+        env=env, universal_newlines=True)
+    # The actual backend will depend on what's installed, but at least tkagg is
+    # present.
+    assert backend.strip().lower() != "agg"
