@@ -1,15 +1,11 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
 import warnings
 
 import numpy as np
-
-from matplotlib.testing.decorators import image_comparison, knownfailureif
-import matplotlib.pyplot as plt
-from nose.tools import assert_raises
 from numpy.testing import assert_array_equal
+import pytest
+
+from matplotlib.testing.decorators import image_comparison
+import matplotlib.pyplot as plt
 from matplotlib.offsetbox import AnchoredOffsetbox, DrawingArea
 from matplotlib.patches import Rectangle
 
@@ -22,18 +18,17 @@ def example_plot(ax, fontsize=12):
     ax.set_title('Title', fontsize=fontsize)
 
 
-@image_comparison(baseline_images=['tight_layout1'])
+@image_comparison(['tight_layout1'])
 def test_tight_layout1():
     'Test tight_layout for a single subplot'
-    fig = plt.figure()
-    ax = fig.add_subplot(111)
+    fig, ax = plt.subplots()
     example_plot(ax, fontsize=24)
     plt.tight_layout()
 
 
-@image_comparison(baseline_images=['tight_layout2'])
+@image_comparison(['tight_layout2'])
 def test_tight_layout2():
-    'Test tight_layout for mutiple subplots'
+    'Test tight_layout for multiple subplots'
     fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(nrows=2, ncols=2)
     example_plot(ax1)
     example_plot(ax2)
@@ -42,57 +37,42 @@ def test_tight_layout2():
     plt.tight_layout()
 
 
-@image_comparison(baseline_images=['tight_layout3'])
+@image_comparison(['tight_layout3'])
 def test_tight_layout3():
-    'Test tight_layout for mutiple subplots'
-
-    fig = plt.figure()
-
+    'Test tight_layout for multiple subplots'
     ax1 = plt.subplot(221)
     ax2 = plt.subplot(223)
     ax3 = plt.subplot(122)
-
     example_plot(ax1)
     example_plot(ax2)
     example_plot(ax3)
-
     plt.tight_layout()
 
 
-@image_comparison(baseline_images=['tight_layout4'],
-                  freetype_version=('2.4.5', '2.4.9'))
+@image_comparison(['tight_layout4'], freetype_version=('2.5.5', '2.6.1'))
 def test_tight_layout4():
     'Test tight_layout for subplot2grid'
-
-    fig = plt.figure()
-
     ax1 = plt.subplot2grid((3, 3), (0, 0))
     ax2 = plt.subplot2grid((3, 3), (0, 1), colspan=2)
     ax3 = plt.subplot2grid((3, 3), (1, 0), colspan=2, rowspan=2)
     ax4 = plt.subplot2grid((3, 3), (1, 2), rowspan=2)
-
     example_plot(ax1)
     example_plot(ax2)
     example_plot(ax3)
     example_plot(ax4)
-
     plt.tight_layout()
 
 
-@image_comparison(baseline_images=['tight_layout5'])
+@image_comparison(['tight_layout5'])
 def test_tight_layout5():
     'Test tight_layout for image'
-
-    fig = plt.figure()
-
     ax = plt.subplot(111)
     arr = np.arange(100).reshape((10, 10))
     ax.imshow(arr, interpolation="none")
-
     plt.tight_layout()
 
 
-@image_comparison(baseline_images=['tight_layout6'])
+@image_comparison(['tight_layout6'])
 def test_tight_layout6():
     'Test tight_layout for gridspec'
 
@@ -136,12 +116,11 @@ def test_tight_layout6():
                          h_pad=0.45)
 
 
-@image_comparison(baseline_images=['tight_layout7'])
+@image_comparison(['tight_layout7'])
 def test_tight_layout7():
     # tight layout with left and right titles
-    fig = plt.figure()
     fontsize = 24
-    ax = fig.add_subplot(111)
+    fig, ax = plt.subplots()
     ax.plot([1, 2])
     ax.locator_params(nbins=3)
     ax.set_xlabel('x-label', fontsize=fontsize)
@@ -151,7 +130,7 @@ def test_tight_layout7():
     plt.tight_layout()
 
 
-@image_comparison(baseline_images=['tight_layout8'])
+@image_comparison(['tight_layout8'])
 def test_tight_layout8():
     'Test automatic use of tight_layout'
     fig = plt.figure()
@@ -160,17 +139,15 @@ def test_tight_layout8():
     example_plot(ax, fontsize=24)
 
 
-@image_comparison(baseline_images=['tight_layout9'])
+@image_comparison(['tight_layout9'])
 def test_tight_layout9():
-    # Test tight_layout for non-visible suplots
+    # Test tight_layout for non-visible subplots
     # GH 8244
     f, axarr = plt.subplots(2, 2)
     axarr[1][1].set_visible(False)
     plt.tight_layout()
 
 
-# The following test is misleading when the text is removed.
-@image_comparison(baseline_images=['outward_ticks'], remove_text=False)
 def test_outward_ticks():
     'Test automatic use of tight_layout'
     fig = plt.figure()
@@ -181,8 +158,6 @@ def test_outward_ticks():
         tickdir='out', length=32, width=3, tick1On=True, which='minor')
     ax.yaxis.set_tick_params(
         tickdir='out', length=32, width=3, tick1On=True, which='minor')
-    # The following minor ticks are not labelled, and they
-    # are drawn over the major ticks and labels--ugly!
     ax.xaxis.set_ticks([0], minor=True)
     ax.yaxis.set_ticks([0], minor=True)
     ax = fig.add_subplot(222)
@@ -195,6 +170,15 @@ def test_outward_ticks():
     ax.xaxis.set_tick_params(tickdir='out', length=32, width=3)
     ax.yaxis.set_tick_params(tickdir='out', length=32, width=3)
     plt.tight_layout()
+    # These values were obtained after visual checking that they correspond
+    # to a tight layouting that did take the ticks into account.
+    ans = [[[0.091, 0.607], [0.433, 0.933]],
+           [[0.579, 0.607], [0.922, 0.933]],
+           [[0.091, 0.140], [0.433, 0.466]],
+           [[0.579, 0.140], [0.922, 0.466]]]
+    for nn, ax in enumerate(fig.axes):
+        assert_array_equal(np.round(ax.get_position().get_points(), 3),
+                           ans[nn])
 
 
 def add_offsetboxes(ax, size=10, margin=.1, color='black'):
@@ -216,7 +200,7 @@ def add_offsetboxes(ax, size=10, margin=.1, color='black'):
         da.add_artist(background)
 
         anchored_box = AnchoredOffsetbox(
-            loc=10,
+            loc='center',
             child=da,
             pad=0.,
             frameon=False,
@@ -227,8 +211,7 @@ def add_offsetboxes(ax, size=10, margin=.1, color='black'):
     return anchored_box
 
 
-@image_comparison(baseline_images=['tight_layout_offsetboxes1',
-                                   'tight_layout_offsetboxes2'])
+@image_comparison(['tight_layout_offsetboxes1', 'tight_layout_offsetboxes2'])
 def test_tight_layout_offsetboxes():
     # 1.
     # - Create 4 subplots
@@ -266,3 +249,63 @@ def test_tight_layout_offsetboxes():
                 child.set_visible(False)
 
     plt.tight_layout()
+
+
+def test_empty_layout():
+    """Test that tight layout doesn't cause an error when there are no axes."""
+    fig = plt.gcf()
+    fig.tight_layout()
+
+
+@pytest.mark.parametrize("label", ["xlabel", "ylabel"])
+def test_verybig_decorators(label):
+    """Test that warning emitted when xlabel/ylabel too big."""
+    fig, ax = plt.subplots(figsize=(3, 2))
+    ax.set(**{label: 'a' * 100})
+    with pytest.warns(UserWarning):
+        fig.tight_layout()
+
+
+def test_big_decorators_horizontal():
+    "Test that warning emitted when xlabel too big"
+    fig, axs = plt.subplots(1, 2, figsize=(3, 2))
+    axs[0].set_xlabel('a' * 30)
+    axs[1].set_xlabel('b' * 30)
+    with pytest.warns(UserWarning):
+        fig.tight_layout()
+
+
+def test_big_decorators_vertical():
+    "Test that warning emitted when xlabel too big"
+    fig, axs = plt.subplots(2, 1, figsize=(3, 2))
+    axs[0].set_ylabel('a' * 20)
+    axs[1].set_ylabel('b' * 20)
+    with pytest.warns(UserWarning):
+        fig.tight_layout()
+
+
+def test_badsubplotgrid():
+    # test that we get warning for mismatched subplot grids, not than an error
+    plt.subplot2grid((4, 5), (0, 0))
+    # this is the bad entry:
+    plt.subplot2grid((5, 5), (0, 3), colspan=3, rowspan=5)
+    with pytest.warns(UserWarning):
+        plt.tight_layout()
+
+
+def test_collapsed():
+    # test that if a call to tight_layout will collapses the axes that
+    # it does not get applied:
+    fig, ax = plt.subplots(tight_layout=True)
+    ax.set_xlim([0, 1])
+    ax.set_ylim([0, 1])
+
+    ax.annotate('BIG LONG STRING', xy=(1.25, 2), xytext=(10.5, 1.75),)
+    p1 = ax.get_position()
+    with pytest.warns(UserWarning):
+        plt.tight_layout()
+        p2 = ax.get_position()
+        assert p1.width == p2.width
+    # test that passing a rect doesn't crash...
+    with pytest.warns(UserWarning):
+        plt.tight_layout(rect=[0, 0, 0.8, 0.8])
