@@ -8,24 +8,18 @@ multiple axes at drawing time.
     and vertical lists of sizes that the division will be based on. You
     then use the new_locator method, whose return value is a callable
     object that can be used to set the axes_locator of the axes.
-
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-from six.moves import map
-
-import matplotlib.transforms as mtransforms
 
+from matplotlib import cbook
 from matplotlib.axes import SubplotBase
-
+from matplotlib.gridspec import SubplotSpec, GridSpec
+import matplotlib.transforms as mtransforms
 from . import axes_size as Size
 
 
-class Divider(object):
+class Divider:
     """
-    This is the class that is used calculates the axes position. It
+    This class calculates the axes position. It
     divides the given rectangular area into several
     sub-rectangles. You initialize the divider by setting the
     horizontal and vertical lists of sizes
@@ -38,20 +32,21 @@ class Divider(object):
     def __init__(self, fig, pos, horizontal, vertical,
                  aspect=None, anchor="C"):
         """
-        :param fig: matplotlib figure
-        :param pos: position (tuple of 4 floats) of the rectangle that
-                    will be divided.
-        :param horizontal: list of sizes
-                    (:mod:`~mpl_toolkits.axes_grid.axes_size`)
-                    for horizontal division
-        :param vertical: list of sizes
-                    (:mod:`~mpl_toolkits.axes_grid.axes_size`)
-                    for vertical division
-        :param aspect: if True, the overall rectangular area is reduced
-                    so that the relative part of the horizontal and
-                    vertical scales have the same scale.
-        :param anchor: Determine how the reduced rectangle is placed
-                       when aspect is True.
+        Parameters
+        ----------
+        fig : Figure
+        pos : tuple of 4 floats
+            position of the rectangle that will be divided
+        horizontal : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
+            sizes for horizontal division
+        vertical : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
+            sizes for vertical division
+        aspect : bool
+            if True, the overall rectangular area is reduced
+            so that the relative part of the horizontal and
+            vertical scales have the same scale.
+        anchor : {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W'}
+            placement of the reduced rectangle when *aspect* is True
         """
 
         self._fig = fig
@@ -96,22 +91,19 @@ class Divider(object):
 
     @staticmethod
     def _calc_offsets(l, k):
-
         offsets = [0.]
-
-        #for s in l:
         for _rs, _as in l:
-            #_rs, _as = s.get_size(renderer)
             offsets.append(offsets[-1] + _rs*k + _as)
-
         return offsets
 
     def set_position(self, pos):
         """
         set the position of the rectangle.
 
-        :param pos: position (tuple of 4 floats) of the rectangle that
-                    will be divided.
+        Parameters
+        ----------
+        pos : tuple of 4 floats
+            position of the rectangle that will be divided
         """
         self._pos = pos
 
@@ -121,7 +113,10 @@ class Divider(object):
 
     def set_anchor(self, anchor):
         """
-        :param anchor: anchor position
+        Parameters
+        ----------
+        anchor : {'C', 'SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W'}
+            anchor position
 
           =====  ============
           value  description
@@ -138,11 +133,9 @@ class Divider(object):
           =====  ============
 
         """
-        if anchor in mtransforms.Bbox.coefs or len(anchor) == 2:
-            self._anchor = anchor
-        else:
-            raise ValueError('argument must be among %s' %
-                                ', '.join(mtransforms.BBox.coefs.keys()))
+        if len(anchor) != 2:
+            cbook._check_in_list(mtransforms.Bbox.coefs, anchor=anchor)
+        self._anchor = anchor
 
     def get_anchor(self):
         "return the anchor"
@@ -150,11 +143,10 @@ class Divider(object):
 
     def set_horizontal(self, h):
         """
-        :param horizontal: list of sizes
-                    (:mod:`~mpl_toolkits.axes_grid.axes_size`)
-                    for horizontal division
-
-        .
+        Parameters
+        ----------
+        h : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
+            sizes for horizontal division
         """
         self._horizontal = h
 
@@ -164,11 +156,10 @@ class Divider(object):
 
     def set_vertical(self, v):
         """
-        :param horizontal: list of sizes
-                    (:mod:`~mpl_toolkits.axes_grid.axes_size`)
-                    for horizontal division
-
-        .
+        Parameters
+        ----------
+        v : list of :mod:`~mpl_toolkits.axes_grid.axes_size`
+            sizes for vertical division
         """
         self._vertical = v
 
@@ -178,7 +169,9 @@ class Divider(object):
 
     def set_aspect(self, aspect=False):
         """
-        :param anchor: True or False
+        Parameters
+        ----------
+        aspect : bool
         """
         self._aspect = aspect
 
@@ -200,13 +193,17 @@ class Divider(object):
 
     def locate(self, nx, ny, nx1=None, ny1=None, axes=None, renderer=None):
         """
-
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        nx, nx1 : int
+            Integers specifying the column-position of the
+            cell. When *nx1* is None, a single *nx*-th column is
+            specified. Otherwise location of columns spanning between *nx*
+            to *nx1* (but excluding *nx1*-th column) is specified.
+        ny, ny1 : int
+            Same as *nx* and *nx1*, but for row positions.
+        axes
+        renderer
         """
 
         figW, figH = self._fig.get_size_inches()
@@ -246,21 +243,23 @@ class Divider(object):
 
     def new_locator(self, nx, ny, nx1=None, ny1=None):
         """
-        returns a new locator
+        Returns a new locator
         (:class:`mpl_toolkits.axes_grid.axes_divider.AxesLocator`) for
         specified cell.
 
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        nx, nx1 : int
+            Integers specifying the column-position of the
+            cell. When *nx1* is None, a single *nx*-th column is
+            specified. Otherwise location of columns spanning between *nx*
+            to *nx1* (but excluding *nx1*-th column) is specified.
+        ny, ny1 : int
+            Same as *nx* and *nx1*, but for row positions.
         """
         return AxesLocator(self, nx, ny, nx1, ny1)
 
     def append_size(self, position, size):
-
         if position == "left":
             self._horizontal.insert(0, size)
             self._xrefindex += 1
@@ -272,8 +271,8 @@ class Divider(object):
         elif position == "top":
             self._vertical.append(size)
         else:
-            raise ValueError("the position must be one of left," +
-                             " right, bottom, or top")
+            cbook._check_in_list(["left", "right", "bottom", "top"],
+                                 position=position)
 
     def add_auto_adjustable_area(self,
                                  use_axes, pad=0.1,
@@ -289,21 +288,23 @@ class Divider(object):
             self.append_size(d, padded_size)
 
 
-class AxesLocator(object):
+class AxesLocator:
     """
     A simple callable object, initialized with AxesDivider class,
     returns the position and size of the given cell.
     """
     def __init__(self, axes_divider, nx, ny, nx1=None, ny1=None):
         """
-        :param axes_divider: An instance of AxesDivider class.
-
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        axes_divider : AxesDivider
+        nx, nx1 : int
+            Integers specifying the column-position of the
+            cell. When *nx1* is None, a single *nx*-th column is
+            specified. Otherwise location of columns spanning between *nx*
+            to *nx1* (but excluding *nx1*-th column) is specified.
+        ny, ny1 : int
+            Same as *nx* and *nx1*, but for row positions.
         """
         self._axes_divider = axes_divider
 
@@ -339,26 +340,27 @@ class AxesLocator(object):
             return None
 
 
-from matplotlib.gridspec import SubplotSpec, GridSpec
-
-
 class SubplotDivider(Divider):
     """
     The Divider class whose rectangle area is specified as a subplot geometry.
     """
 
-    def __init__(self, fig, *args, **kwargs):
+    def __init__(self, fig, *args, horizontal=None, vertical=None,
+                 aspect=None, anchor='C'):
         """
-        *fig* is a :class:`matplotlib.figure.Figure` instance.
+        Parameters
+        ----------
+        fig : `matplotlib.figure.Figure`
 
-        *args* is the tuple (*numRows*, *numCols*, *plotNum*), where
-        the array of subplots in the figure has dimensions *numRows*,
-        *numCols*, and where *plotNum* is the number of the subplot
-        being created.  *plotNum* starts at 1 in the upper left
-        corner and increases to the right.
+        *args : tuple (*nrows*, *ncols*, *index*) or int
+            The array of subplots in the figure has dimensions ``(nrows,
+            ncols)``, and *index* is the index of the subplot being created.
+            *index* starts at 1 in the upper left corner and increases to the
+            right.
 
-        If *numRows* <= *numCols* <= *plotNum* < 10, *args* can be the
-        decimal integer *numRows* * 100 + *numCols* * 10 + *plotNum*.
+            If *nrows*, *ncols*, and *index* are all single digit numbers, then
+            *args* can be passed as a single 3-digit number (e.g. 234 for
+            (2, 3, 4)).
         """
 
         self.figure = fig
@@ -369,7 +371,7 @@ class SubplotDivider(Divider):
             else:
                 try:
                     s = str(int(args[0]))
-                    rows, cols, num = list(map(int, s))
+                    rows, cols, num = map(int, s)
                 except ValueError:
                     raise ValueError(
                         'Single argument to subplot must be a 3-digit integer')
@@ -386,11 +388,11 @@ class SubplotDivider(Divider):
                 self._subplotspec = GridSpec(rows, cols)[int(num)-1]
                 # num - 1 for converting from MATLAB to python indexing
         else:
-            raise ValueError('Illegal argument(s) to subplot: %s' % (args,))
+            raise ValueError(f'Illegal argument(s) to subplot: {args}')
 
         # total = rows*cols
         # num -= 1    # convert from matlab to python indexing
-        #             # ie num in range(0,total)
+        #             # i.e., num in range(0, total)
         # if num >= total:
         #     raise ValueError( 'Subplot number exceeds total subplots')
         # self._rows = rows
@@ -404,15 +406,7 @@ class SubplotDivider(Divider):
 
         pos = self.figbox.bounds
 
-        horizontal = kwargs.pop("horizontal", [])
-        vertical = kwargs.pop("vertical", [])
-        aspect = kwargs.pop("aspect", None)
-        anchor = kwargs.pop("anchor", "C")
-
-        if kwargs:
-            raise Exception("")
-
-        Divider.__init__(self, fig, pos, horizontal, vertical,
+        Divider.__init__(self, fig, pos, horizontal or [], vertical or [],
                          aspect=aspect, anchor=anchor)
 
     def get_position(self):
@@ -453,28 +447,27 @@ class SubplotDivider(Divider):
     #                                                figW, figH)
 
     def update_params(self):
-        'update the subplot position from fig.subplotpars'
-
+        """Update the subplot position from fig.subplotpars."""
         self.figbox = self.get_subplotspec().get_position(self.figure)
 
     def get_geometry(self):
-        'get the subplot geometry, eg 2,2,3'
+        """Get the subplot geometry, e.g., (2, 2, 3)."""
         rows, cols, num1, num2 = self.get_subplotspec().get_geometry()
         return rows, cols, num1+1  # for compatibility
 
     # COVERAGE NOTE: Never used internally or from examples
     def change_geometry(self, numrows, numcols, num):
-        'change subplot geometry, e.g., from 1,1,1 to 2,2,3'
+        """Change subplot geometry, e.g., from (1, 1, 1) to (2, 2, 3)."""
         self._subplotspec = GridSpec(numrows, numcols)[num-1]
         self.update_params()
         self.set_position(self.figbox)
 
     def get_subplotspec(self):
-        'get the SubplotSpec instance'
+        """Get the SubplotSpec instance."""
         return self._subplotspec
 
     def set_subplotspec(self, subplotspec):
-        'set the SubplotSpec instance'
+        """Set the SubplotSpec instance."""
         self._subplotspec = subplotspec
 
 
@@ -485,7 +478,11 @@ class AxesDivider(Divider):
 
     def __init__(self, axes, xref=None, yref=None):
         """
-        :param axes: axes
+        Parameters
+        ----------
+        axes : :class:`~matplotlib.axes.Axes`
+        xref
+        yref
         """
         self._axes = axes
         if xref is None:
@@ -501,120 +498,118 @@ class AxesDivider(Divider):
                          horizontal=[self._xref], vertical=[self._yref],
                          aspect=None, anchor="C")
 
-    def _get_new_axes(self, **kwargs):
+    def _get_new_axes(self, *, axes_class=None, **kwargs):
         axes = self._axes
-
-        axes_class = kwargs.pop("axes_class", None)
-
         if axes_class is None:
             if isinstance(axes, SubplotBase):
                 axes_class = axes._axes_class
             else:
                 axes_class = type(axes)
-
-        ax = axes_class(axes.get_figure(),
-                        axes.get_position(original=True), **kwargs)
-
-        return ax
+        return axes_class(axes.get_figure(), axes.get_position(original=True),
+                          **kwargs)
 
     def new_horizontal(self, size, pad=None, pack_start=False, **kwargs):
         """
         Add a new axes on the right (or left) side of the main axes.
 
-        :param size: A width of the axes.
-          A :mod:`~mpl_toolkits.axes_grid.axes_size`
-          instance or if float or string is given, *from_any*
-          function is used to create one, with *ref_size* set to AxesX instance
-          of the current axes.
-        :param pad: pad between the axes. It takes same argument as *size*.
-        :param pack_start: If False, the new axes is appended at the end
-          of the list, i.e., it became the right-most axes. If True, it is
-          inserted at the start of the list, and becomes the left-most axes.
-
-        All extra keywords arguments are passed to the created axes.
-        If *axes_class* is given, the new axes will be created as an
-        instance of the given class. Otherwise, the same class of the
-        main axes will be used.
+        Parameters
+        ----------
+        size : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
+            A width of the axes. If float or string is given, *from_any*
+            function is used to create the size, with *ref_size* set to AxesX
+            instance of the current axes.
+        pad : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
+            Pad between the axes. It takes same argument as *size*.
+        pack_start : bool
+            If False, the new axes is appended at the end
+            of the list, i.e., it became the right-most axes. If True, it is
+            inserted at the start of the list, and becomes the left-most axes.
+        **kwargs
+            All extra keywords arguments are passed to the created axes.
+            If *axes_class* is given, the new axes will be created as an
+            instance of the given class. Otherwise, the same class of the
+            main axes will be used.
         """
-
+        if pad is None:
+            cbook.warn_deprecated(
+                "3.2", message="In a future version, 'pad' will default to "
+                "rcParams['figure.subplot.wspace'].  Set pad=0 to keep the "
+                "old behavior.")
         if pad:
             if not isinstance(pad, Size._Base):
-                pad = Size.from_any(pad,
-                                    fraction_ref=self._xref)
+                pad = Size.from_any(pad, fraction_ref=self._xref)
             if pack_start:
                 self._horizontal.insert(0, pad)
                 self._xrefindex += 1
             else:
                 self._horizontal.append(pad)
-
         if not isinstance(size, Size._Base):
-            size = Size.from_any(size,
-                                 fraction_ref=self._xref)
-
+            size = Size.from_any(size, fraction_ref=self._xref)
         if pack_start:
             self._horizontal.insert(0, size)
             self._xrefindex += 1
             locator = self.new_locator(nx=0, ny=self._yrefindex)
         else:
             self._horizontal.append(size)
-            locator = self.new_locator(nx=len(self._horizontal)-1, ny=self._yrefindex)
-
+            locator = self.new_locator(
+                nx=len(self._horizontal) - 1, ny=self._yrefindex)
         ax = self._get_new_axes(**kwargs)
         ax.set_axes_locator(locator)
-
         return ax
 
     def new_vertical(self, size, pad=None, pack_start=False, **kwargs):
         """
         Add a new axes on the top (or bottom) side of the main axes.
 
-        :param size: A height of the axes.
-          A :mod:`~mpl_toolkits.axes_grid.axes_size`
-          instance or if float or string is given, *from_any*
-          function is used to create one, with *ref_size* set to AxesX instance
-          of the current axes.
-        :param pad: pad between the axes. It takes same argument as *size*.
-        :param pack_start: If False, the new axes is appended at the end
-          of the list, i.e., it became the top-most axes. If True, it is
-          inserted at the start of the list, and becomes the bottom-most axes.
-
-        All extra keywords arguments are passed to the created axes.
-        If *axes_class* is given, the new axes will be created as an
-        instance of the given class. Otherwise, the same class of the
-        main axes will be used.
+        Parameters
+        ----------
+        size : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
+            A height of the axes. If float or string is given, *from_any*
+            function is used to create the size, with *ref_size* set to AxesX
+            instance of the current axes.
+        pad : :mod:`~mpl_toolkits.axes_grid.axes_size` or float or str
+            Pad between the axes. It takes same argument as *size*.
+        pack_start : bool
+            If False, the new axes is appended at the end
+            of the list, i.e., it became the right-most axes. If True, it is
+            inserted at the start of the list, and becomes the left-most axes.
+        **kwargs
+            All extra keywords arguments are passed to the created axes.
+            If *axes_class* is given, the new axes will be created as an
+            instance of the given class. Otherwise, the same class of the
+            main axes will be used.
         """
-
+        if pad is None:
+            cbook.warn_deprecated(
+                "3.2", message="In a future version, 'pad' will default to "
+                "rcParams['figure.subplot.hspace'].  Set pad=0 to keep the "
+                "old behavior.")
         if pad:
             if not isinstance(pad, Size._Base):
-                pad = Size.from_any(pad,
-                                    fraction_ref=self._yref)
+                pad = Size.from_any(pad, fraction_ref=self._yref)
             if pack_start:
                 self._vertical.insert(0, pad)
                 self._yrefindex += 1
             else:
                 self._vertical.append(pad)
-
         if not isinstance(size, Size._Base):
-            size = Size.from_any(size,
-                                 fraction_ref=self._yref)
-
+            size = Size.from_any(size, fraction_ref=self._yref)
         if pack_start:
             self._vertical.insert(0, size)
             self._yrefindex += 1
             locator = self.new_locator(nx=self._xrefindex, ny=0)
         else:
             self._vertical.append(size)
-            locator = self.new_locator(nx=self._xrefindex, ny=len(self._vertical)-1)
-
+            locator = self.new_locator(
+                nx=self._xrefindex, ny=len(self._vertical)-1)
         ax = self._get_new_axes(**kwargs)
         ax.set_axes_locator(locator)
-
         return ax
 
     def append_axes(self, position, size, pad=None, add_to_figure=True,
                     **kwargs):
         """
-        create an axes at the given *position* with the same height
+        Create an axes at the given *position* with the same height
         (or width) of the main axes.
 
          *position*
@@ -622,7 +617,6 @@ class AxesDivider(Divider):
 
          *size* and *pad* should be axes_grid.axes_size compatible.
         """
-
         if position == "left":
             ax = self.new_horizontal(size, pad, pack_start=True, **kwargs)
         elif position == "right":
@@ -632,9 +626,8 @@ class AxesDivider(Divider):
         elif position == "top":
             ax = self.new_vertical(size, pad, pack_start=False, **kwargs)
         else:
-            raise ValueError("the position must be one of left," +
-                             " right, bottom, or top")
-
+            cbook._check_in_list(["left", "right", "bottom", "top"],
+                                 position=position)
         if add_to_figure:
             self._fig.add_axes(ax)
         return ax
@@ -706,25 +699,24 @@ class HBoxDivider(SubplotDivider):
     @staticmethod
     def _calc_offsets(appended_sizes, karray):
         offsets = [0.]
-
-        #for s in l:
         for (r, a), k in zip(appended_sizes, karray):
             offsets.append(offsets[-1] + r*k + a)
-
         return offsets
 
     def new_locator(self, nx, nx1=None):
         """
-        returns a new locator
-        (:class:`mpl_toolkits.axes_grid.axes_divider.AxesLocator`) for
-        specified cell.
+        Create a new `~mpl_toolkits.axes_grid.axes_divider.AxesLocator` for
+        the specified cell.
 
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        nx, nx1 : int
+            Integers specifying the column-position of the
+            cell. When *nx1* is None, a single *nx*-th column is
+            specified. Otherwise location of columns spanning between *nx*
+            to *nx1* (but excluding *nx1*-th column) is specified.
+        ny, ny1 : int
+            Same as *nx* and *nx1*, but for row positions.
         """
         return AxesLocator(self, nx, 0, nx1, None)
 
@@ -732,13 +724,16 @@ class HBoxDivider(SubplotDivider):
                 y_equivalent_sizes, x_appended_sizes,
                 figW, figH):
         """
-
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        x
+        y
+        w
+        h
+        y_equivalent_sizes
+        x_appended_sizes
+        figW
+        figH
         """
 
         equivalent_sizes = y_equivalent_sizes
@@ -764,13 +759,18 @@ class HBoxDivider(SubplotDivider):
 
     def locate(self, nx, ny, nx1=None, ny1=None, axes=None, renderer=None):
         """
-
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        axes_divider : AxesDivider
+        nx, nx1 : int
+            Integers specifying the column-position of the
+            cell. When *nx1* is None, a single *nx*-th column is
+            specified. Otherwise location of columns spanning between *nx*
+            to *nx1* (but excluding *nx1*-th column) is specified.
+        ny, ny1 : int
+            Same as *nx* and *nx1*, but for row positions.
+        axes
+        renderer
         """
 
         figW, figH = self._fig.get_size_inches()
@@ -797,28 +797,33 @@ class VBoxDivider(HBoxDivider):
 
     def new_locator(self, ny, ny1=None):
         """
-        returns a new locator
-        (:class:`mpl_toolkits.axes_grid.axes_divider.AxesLocator`) for
-        specified cell.
+        Create a new `~mpl_toolkits.axes_grid.axes_divider.AxesLocator` for
+        the specified cell.
 
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        ny, ny1 : int
+            Integers specifying the row-position of the
+            cell. When *ny1* is None, a single *ny*-th row is
+            specified. Otherwise location of rows spanning between *ny*
+            to *ny1* (but excluding *ny1*-th row) is specified.
         """
         return AxesLocator(self, 0, ny, None, ny1)
 
     def locate(self, nx, ny, nx1=None, ny1=None, axes=None, renderer=None):
         """
-
-        :param nx, nx1: Integers specifying the column-position of the
-          cell. When nx1 is None, a single nx-th column is
-          specified. Otherwise location of columns spanning between nx
-          to nx1 (but excluding nx1-th column) is specified.
-
-        :param ny, ny1: same as nx and nx1, but for row positions.
+        Parameters
+        ----------
+        axes_divider : AxesDivider
+        nx, nx1 : int
+            Integers specifying the column-position of the
+            cell. When *nx1* is None, a single *nx*-th column is
+            specified. Otherwise location of columns spanning between *nx*
+            to *nx1* (but excluding *nx1*-th column) is specified.
+        ny, ny1 : int
+            Same as *nx* and *nx1*, but for row positions.
+        axes
+        renderer
         """
 
         figW, figH = self._fig.get_size_inches()
@@ -839,69 +844,7 @@ class VBoxDivider(HBoxDivider):
         return mtransforms.Bbox.from_bounds(x1, y1, w1, h1)
 
 
-class LocatableAxesBase:
-    def __init__(self, *kl, **kw):
-
-        self._axes_class.__init__(self, *kl, **kw)
-
-        self._locator = None
-        self._locator_renderer = None
-
-    def set_axes_locator(self, locator):
-        self._locator = locator
-
-    def get_axes_locator(self):
-        return self._locator
-
-    def apply_aspect(self, position=None):
-
-        if self.get_axes_locator() is None:
-            self._axes_class.apply_aspect(self, position)
-        else:
-            pos = self.get_axes_locator()(self, self._locator_renderer)
-            self._axes_class.apply_aspect(self, position=pos)
-
-    def draw(self, renderer=None, inframe=False):
-
-        self._locator_renderer = renderer
-
-        self._axes_class.draw(self, renderer, inframe)
-
-    def _make_twin_axes(self, *kl, **kwargs):
-        """
-        Need to overload so that twinx/twiny will work with
-        these axes.
-        """
-        ax2 = type(self)(self.figure, self.get_position(True), *kl, **kwargs)
-        ax2.set_axes_locator(self.get_axes_locator())
-        self.figure.add_axes(ax2)
-        return ax2
-
-_locatableaxes_classes = {}
-
-
-def locatable_axes_factory(axes_class):
-
-    new_class = _locatableaxes_classes.get(axes_class)
-    if new_class is None:
-        new_class = type(str("Locatable%s" % (axes_class.__name__)),
-                         (LocatableAxesBase, axes_class),
-                         {'_axes_class': axes_class})
-
-        _locatableaxes_classes[axes_class] = new_class
-
-    return new_class
-
-#if hasattr(maxes.Axes, "get_axes_locator"):
-#    LocatableAxes = maxes.Axes
-#else:
-
-
 def make_axes_locatable(axes):
-    if not hasattr(axes, "set_axes_locator"):
-        new_class = locatable_axes_factory(type(axes))
-        axes.__class__ = new_class
-
     divider = AxesDivider(axes)
     locator = divider.new_locator(nx=0, ny=0)
     axes.set_axes_locator(locator)
@@ -921,7 +864,3 @@ def make_axes_area_auto_adjustable(ax,
 
     divider.add_auto_adjustable_area(use_axes=use_axes, pad=pad,
                                      adjust_dirs=adjust_dirs)
-
-#from matplotlib.axes import Axes
-from .mpl_axes import Axes
-LocatableAxes = locatable_axes_factory(Axes)
