@@ -1,4 +1,4 @@
-'''
+"""
 Colorbar toolkit with two classes and a function:
 
     :class:`ColorbarBase`
@@ -16,21 +16,15 @@ Colorbar toolkit with two classes and a function:
 The :meth:`~matplotlib.figure.Figure.colorbar` method uses :func:`make_axes`
 and :class:`Colorbar`; the :func:`~matplotlib.pyplot.colorbar` function
 is a thin wrapper over :meth:`~matplotlib.figure.Figure.colorbar`.
-
-'''
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-from matplotlib.externals import six
-from matplotlib.externals.six.moves import xrange, zip
+"""
 
 import numpy as np
 import matplotlib as mpl
+from matplotlib import cbook
 import matplotlib.colors as colors
 import matplotlib.cm as cm
 from matplotlib import docstring
 import matplotlib.ticker as ticker
-import matplotlib.cbook as cbook
 import matplotlib.collections as collections
 import matplotlib.contour as contour
 from matplotlib.path import Path
@@ -38,6 +32,10 @@ from matplotlib.patches import PathPatch
 from matplotlib.transforms import Bbox
 
 
+cbook.warn_deprecated(
+    "3.2", name=__name__, obj_type="module", alternative="matplotlib.colorbar")
+
+
 make_axes_kw_doc = '''
 
     ============= ====================================================
@@ -76,8 +74,8 @@ colormap_kw_doc = '''
                   used. An alternative
                   :class:`~matplotlib.ticker.Formatter` object may be
                   given instead.
-    *drawedges*   [ False | True ] If true, draw lines at color
-                  boundaries.
+    *drawedges*   bool
+                  Whether to draw lines at color boundaries.
     ===========   ====================================================
 
     The following will probably be useful only in the context of
@@ -132,9 +130,9 @@ keyword arguments:
 Additional keyword arguments are of two kinds:
 
   axes properties:
-%s
+    %s
   colorbar properties:
-%s
+    %s
 
 If *mappable* is a :class:`~matplotlib.contours.ContourSet`, its *extend*
 kwarg is included automatically.
@@ -158,9 +156,9 @@ segments::
     cbar.solids.set_edgecolor("face")
     draw()
 
-However this has negative consequences in other circumstances. Particularly with
-semi transparent images (alpha < 1) and colorbar extensions and is not enabled
-by default see (issue #1188).
+However this has negative consequences in other circumstances. Particularly
+with semi transparent images (alpha < 1) and colorbar extensions and is not
+enabled by default see (issue #1188).
 
 returns:
     :class:`~matplotlib.colorbar.Colorbar` instance; see also its base class,
@@ -175,10 +173,10 @@ the other hand, the shortest axis has a data limits of [1,2], whose
 unconventional value is to prevent underflow when log scale is used.
 ''' % (make_axes_kw_doc, colormap_kw_doc)
 
-docstring.interpd.update(colorbar_doc=colorbar_doc)
+#docstring.interpd.update(colorbar_doc=colorbar_doc)
 
 
-class CbarAxesLocator(object):
+class CbarAxesLocator:
     """
     CbarAxesLocator is a axes_locator for colorbar axes. It adjust the
     position of the axes to make a room for extended ends, i.e., the
@@ -242,33 +240,26 @@ class CbarAxesLocator(object):
                    (x1, y2)]
 
         if self.orientation == "horizontal":
-            bottom = [(y,x) for (x,y) in bottom]
-            top = [(y,x) for (x,y) in top]
+            bottom = [(y, x) for (x, y) in bottom]
+            top = [(y, x) for (x, y) in top]
 
         return bottom, top
 
-
     def get_path_patch(self):
         """
         get the path for axes patch
         """
         end1, end2 = self.get_end_vertices()
-
         verts = [] + end1 + end2 + end1[:1]
-
         return Path(verts)
 
-
     def get_path_ends(self):
         """
         get the paths for extended ends
         """
-
         end1, end2 = self.get_end_vertices()
-
         return Path(end1), Path(end2)
 
-
     def __call__(self, axes, renderer):
         """
         Return the adjusted position of the axes
@@ -295,13 +286,12 @@ class CbarAxesLocator(object):
         return Bbox.from_bounds(x1, y1, w, h)
 
 
-
 class ColorbarBase(cm.ScalarMappable):
     '''
     Draw a colorbar in an existing axes.
 
     This is a base class for the :class:`Colorbar` class, which is the
-    basis for the :func:`~matplotlib.pyplot.colorbar` method and pylab
+    basis for the :func:`~matplotlib.pyplot.colorbar` method and pyplot
     function.
 
     It is also useful by itself for showing a colormap.  If the *cmap*
@@ -328,26 +318,28 @@ class ColorbarBase(cm.ScalarMappable):
             a LineCollection if *drawedges* is True, otherwise None
 
     Useful public methods are :meth:`set_label` and :meth:`add_lines`.
-
     '''
 
-    def __init__(self, ax, cmap=None,
-                           norm=None,
-                           alpha=1.0,
-                           values=None,
-                           boundaries=None,
-                           orientation='vertical',
-                           extend='neither',
-                           spacing='uniform',  # uniform or proportional
-                           ticks=None,
-                           format=None,
-                           drawedges=False,
-                           filled=True,
-                           ):
+    def __init__(self, ax,
+                 cmap=None,
+                 norm=None,
+                 alpha=1.0,
+                 values=None,
+                 boundaries=None,
+                 orientation='vertical',
+                 extend='neither',
+                 spacing='uniform',  # uniform or proportional
+                 ticks=None,
+                 format=None,
+                 drawedges=False,
+                 filled=True,
+                 ):
         self.ax = ax
 
-        if cmap is None: cmap = cm.get_cmap()
-        if norm is None: norm = colors.Normalize()
+        if cmap is None:
+            cmap = cm.get_cmap()
+        if norm is None:
+            norm = colors.Normalize()
         self.alpha = alpha
         cm.ScalarMappable.__init__(self, cmap=cmap, norm=norm)
         self.values = values
@@ -370,7 +362,6 @@ class ColorbarBase(cm.ScalarMappable):
         else:
             self.cbar_axis = self.ax.xaxis
 
-
         if format is None:
             if isinstance(self.norm, colors.LogNorm):
                 # change both axis for proper aspect
@@ -380,7 +371,7 @@ class ColorbarBase(cm.ScalarMappable):
                 formatter = ticker.LogFormatter()
             else:
                 formatter = None
-        elif cbook.is_string_like(format):
+        elif isinstance(format, str):
             formatter = ticker.FormatStrFormatter(format)
         else:
             formatter = format  # Assume it is a Formatter
@@ -390,21 +381,19 @@ class ColorbarBase(cm.ScalarMappable):
         else:
             self.cbar_axis.set_major_formatter(formatter)
 
-        if cbook.iterable(ticks):
+        if np.iterable(ticks):
             self.cbar_axis.set_ticks(ticks)
         elif ticks is not None:
             self.cbar_axis.set_major_locator(ticks)
         else:
             self._select_locator(formatter)
 
-
         self._config_axes()
 
         self.update_artists()
 
         self.set_label_text('')
 
-
     def _get_colorbar_limits(self):
         """
         initial limits for colorbar range. The returned min, max values
@@ -421,7 +410,6 @@ class ColorbarBase(cm.ScalarMappable):
         else:
             return self.get_clim()
 
-
     def _config_axes(self):
         '''
         Adjust the properties of the axes to be adequate for colorbar display.
@@ -453,8 +441,6 @@ class ColorbarBase(cm.ScalarMappable):
             ax.yaxis.set_label_position('right')
             ax.yaxis.set_ticks_position('right')
 
-
-
     def update_artists(self):
         """
         Update the colorbar associated artists, *filled* and
@@ -466,7 +452,7 @@ class ColorbarBase(cm.ScalarMappable):
 
         X, Y = self._mesh()
         if self.filled:
-            C = self._values[:,np.newaxis]
+            C = self._values[:, np.newaxis]
             self._add_solids(X, Y, C)
 
         ax = self.ax
@@ -478,7 +464,6 @@ class ColorbarBase(cm.ScalarMappable):
             ax.set_xlim(1, 2)
             ax.set_ylim(vmin, vmax)
 
-
     def _add_ends(self):
         """
         Create patches from extended ends and add them to the axes.
@@ -488,9 +473,9 @@ class ColorbarBase(cm.ScalarMappable):
         del self.extension_patch2
 
         path1, path2 = self.ax.get_axes_locator().get_path_ends()
-        fc=mpl.rcParams['axes.facecolor']
-        ec=mpl.rcParams['axes.edgecolor']
-        linewidths=0.5*mpl.rcParams['axes.linewidth']
+        fc = mpl.rcParams['axes.facecolor']
+        ec = mpl.rcParams['axes.edgecolor']
+        linewidths = 0.5 * mpl.rcParams['axes.linewidth']
         self.extension_patch1 = PathPatch(path1,
                                           fc=fc, ec=ec, lw=linewidths,
                                           zorder=2.,
@@ -504,8 +489,6 @@ class ColorbarBase(cm.ScalarMappable):
         self.ax.add_artist(self.extension_patch1)
         self.ax.add_artist(self.extension_patch2)
 
-
-
     def _set_label_text(self):
         """
         set label.
@@ -520,7 +503,6 @@ class ColorbarBase(cm.ScalarMappable):
         self._labelkw = kw
         self._set_label_text()
 
-
     def _edges(self, X, Y):
         '''
         Return the separator line segments; helper for _add_solids.
@@ -529,9 +511,9 @@ class ColorbarBase(cm.ScalarMappable):
         # Using the non-array form of these line segments is much
         # simpler than making them into arrays.
         if self.orientation == 'vertical':
-            return [list(zip(X[i], Y[i])) for i in xrange(1, N-1)]
+            return [list(zip(X[i], Y[i])) for i in range(1, N-1)]
         else:
-            return [list(zip(Y[i], X[i])) for i in xrange(1, N-1)]
+            return [list(zip(Y[i], X[i])) for i in range(1, N-1)]
 
     def _add_solids(self, X, Y, C):
         '''
@@ -542,33 +524,33 @@ class ColorbarBase(cm.ScalarMappable):
 
         if self.extend in ["min", "both"]:
             cc = self.to_rgba([C[0][0]])
-            self.extension_patch1.set_fc(cc[0])
+            self.extension_patch1.set_facecolor(cc[0])
             X, Y, C = X[1:], Y[1:], C[1:]
 
         if self.extend in ["max", "both"]:
             cc = self.to_rgba([C[-1][0]])
-            self.extension_patch2.set_fc(cc[0])
+            self.extension_patch2.set_facecolor(cc[0])
             X, Y, C = X[:-1], Y[:-1], C[:-1]
 
         if self.orientation == 'vertical':
             args = (X, Y, C)
         else:
             args = (np.transpose(Y), np.transpose(X), np.transpose(C))
-        kw = {'cmap':self.cmap, 'norm':self.norm,
-              'shading':'flat', 'alpha':self.alpha,
-              }
 
         del self.solids
         del self.dividers
 
-        col = self.ax.pcolormesh(*args, **kw)
+        col = self.ax.pcolormesh(
+            *args,
+            cmap=self.cmap, norm=self.norm, shading='flat', alpha=self.alpha)
 
         self.solids = col
         if self.drawedges:
-            self.dividers = collections.LineCollection(self._edges(X,Y),
-                              colors=(mpl.rcParams['axes.edgecolor'],),
-                              linewidths=(0.5*mpl.rcParams['axes.linewidth'],),
-                              )
+            self.dividers = collections.LineCollection(
+                self._edges(X, Y),
+                colors=(mpl.rcParams['axes.edgecolor'],),
+                linewidths=(0.5*mpl.rcParams['axes.linewidth'],),
+            )
             self.ax.add_collection(self.dividers)
         else:
             self.dividers = None
@@ -577,22 +559,16 @@ class ColorbarBase(cm.ScalarMappable):
         '''
         Draw lines on the colorbar. It deletes preexisting lines.
         '''
-        del self.lines
-
-        N = len(levels)
-        x = np.array([1.0, 2.0])
-        X, Y = np.meshgrid(x,levels)
+        X, Y = np.meshgrid([1, 2], levels)
         if self.orientation == 'vertical':
-            xy = [list(zip(X[i], Y[i])) for i in xrange(N)]
+            xy = np.stack([X, Y], axis=-1)
         else:
-            xy = [list(zip(Y[i], X[i])) for i in xrange(N)]
-        col = collections.LineCollection(xy, linewidths=linewidths,
-                                         )
+            xy = np.stack([Y, X], axis=-1)
+        col = collections.LineCollection(xy, linewidths=linewidths)
         self.lines = col
         col.set_color(colors)
         self.ax.add_collection(col)
 
-
     def _select_locator(self, formatter):
         '''
         select a suitable locator
@@ -611,11 +587,10 @@ class ColorbarBase(cm.ScalarMappable):
                 locator = ticker.MaxNLocator(nbins=5)
         else:
             b = self._boundaries[self._inside]
-            locator = ticker.FixedLocator(b) #, nbins=10)
+            locator = ticker.FixedLocator(b)
 
         self.cbar_axis.set_major_locator(locator)
 
-
     def _process_values(self, b=None):
         '''
         Set the :attr:`_boundaries` and :attr:`_values` attributes
@@ -627,8 +602,8 @@ class ColorbarBase(cm.ScalarMappable):
         if b is not None:
             self._boundaries = np.asarray(b, dtype=float)
             if self.values is None:
-                self._values = 0.5*(self._boundaries[:-1]
-                                        + self._boundaries[1:])
+                self._values = (self._boundaries[:-1]
+                                + self._boundaries[1:]) / 2
                 if isinstance(self.norm, colors.NoNorm):
                     self._values = (self._values + 0.00001).astype(np.int16)
                 return
@@ -637,7 +612,7 @@ class ColorbarBase(cm.ScalarMappable):
         if self.values is not None:
             self._values = np.array(self.values)
             if self.boundaries is None:
-                b = np.zeros(len(self.values)+1, 'd')
+                b = np.zeros(len(self.values) + 1)
                 b[1:-1] = 0.5*(self._values[:-1] - self._values[1:])
                 b[0] = 2.0*b[1] - b[2]
                 b[-1] = 2.0*b[-2] - b[-3]
@@ -648,26 +623,19 @@ class ColorbarBase(cm.ScalarMappable):
         # Neither boundaries nor values are specified;
         # make reasonable ones based on cmap and norm.
         if isinstance(self.norm, colors.NoNorm):
-            b = self._uniform_y(self.cmap.N+1) * self.cmap.N - 0.5
-            v = np.zeros((len(b)-1,), dtype=np.int16)
-            v = np.arange(self.cmap.N, dtype=np.int16)
-            self._boundaries = b
-            self._values = v
+            self._boundaries = (
+                self._uniform_y(self.cmap.N + 1) * self.cmap.N - 0.5)
+            self._values = np.arange(self.cmap.N, dtype=np.int16)
             return
         elif isinstance(self.norm, colors.BoundaryNorm):
-            b = np.array(self.norm.boundaries)
-            v = np.zeros((len(b)-1,), dtype=float)
-            bi = self.norm.boundaries
-            v = 0.5*(bi[:-1] + bi[1:])
-            self._boundaries = b
-            self._values = v
+            self._boundaries = np.array(self.norm.boundaries)
+            self._values = (self._boundaries[:-1] + self._boundaries[1:]) / 2
             return
         else:
-            b = self._uniform_y(self.cmap.N+1)
+            b = self._uniform_y(self.cmap.N + 1)
 
         self._process_values(b)
 
-
     def _uniform_y(self, N):
         '''
         Return colorbar data coordinates for *N* uniformly
@@ -694,10 +662,9 @@ class ColorbarBase(cm.ScalarMappable):
             y = self._boundaries
         self._y = y
 
-        X, Y = np.meshgrid(x,y)
+        X, Y = np.meshgrid(x, y)
         return X, Y
 
-
     def set_alpha(self, alpha):
         """
         set alpha value.
@@ -707,11 +674,11 @@ class ColorbarBase(cm.ScalarMappable):
 
 class Colorbar(ColorbarBase):
     def __init__(self, ax, mappable, **kw):
-        mappable.autoscale_None() # Ensure mappable.norm.vmin, vmax
-                             # are set when colorbar is called,
-                             # even if mappable.draw has not yet
-                             # been called.  This will not change
-                             # vmin, vmax if they are already set.
+        # Ensure mappable.norm.vmin, vmax are set when colorbar is called, even
+        # if mappable.draw has not yet been called. This will not change vmin,
+        # vmax if they are already set.
+        mappable.autoscale_None()
+
         self.mappable = mappable
         kw['cmap'] = mappable.cmap
         kw['norm'] = mappable.norm
@@ -730,7 +697,6 @@ class Colorbar(ColorbarBase):
         else:
             ColorbarBase.__init__(self, ax, **kw)
 
-
     def add_lines(self, CS):
         '''
         Add the lines from a non-filled
@@ -747,7 +713,6 @@ class Colorbar(ColorbarBase):
         # to make one object track another automatically.
         #tcolors = [col.get_colors()[0] for col in CS.collections]
         #tlinewidths = [col.get_linewidth()[0] for lw in CS.collections]
-        #print 'tlinewidths:', tlinewidths
         ColorbarBase.add_lines(self, CS.levels, tcolors, tlinewidths)
 
     def update_bruteforce(self, mappable):
@@ -761,11 +726,14 @@ class Colorbar(ColorbarBase):
             if not mappable.filled:
                 self.add_lines(mappable)
 
+
 @docstring.Substitution(make_axes_kw_doc)
-def make_axes(parent, **kw):
+def make_axes(parent, *, fraction=0.15, shrink=1.0, aspect=20, **kw):
     '''
     Resize and reposition a parent axes, and return a child
-    axes suitable for a colorbar::
+    axes suitable for a colorbar
+
+    ::
 
         cax, kw = make_axes(parent, **kw)
 
@@ -781,9 +749,6 @@ def make_axes(parent, **kw):
     Returns (cax, kw), the child axes and the reduced kw dictionary.
     '''
     orientation = kw.setdefault('orientation', 'vertical')
-    fraction = kw.pop('fraction', 0.15)
-    shrink = kw.pop('shrink', 1.0)
-    aspect = kw.pop('aspect', 20)
     #pb = transforms.PBox(parent.get_position())
     pb = parent.get_position(original=True).frozen()
     if orientation == 'vertical':
@@ -808,19 +773,20 @@ def make_axes(parent, **kw):
     return cax, kw
 
 
+@docstring.Substitution(colorbar_doc)
 def colorbar(mappable, cax=None, ax=None, **kw):
     """
     Create a colorbar for a ScalarMappable instance.
 
-    Documentation for the pylab thin wrapper:
-    %(colorbar_doc)s
+    Documentation for the pyplot thin wrapper:
+
+    %s
     """
     import matplotlib.pyplot as plt
     if ax is None:
         ax = plt.gca()
     if cax is None:
         cax, kw = make_axes(ax, **kw)
-    cax.hold(True)
     cb = Colorbar(cax, mappable, **kw)
 
     def on_changed(m):
@@ -828,7 +794,7 @@ def colorbar(mappable, cax=None, ax=None, **kw):
         cb.set_clim(m.get_clim())
         cb.update_bruteforce(m)
 
-    cbid = mappable.callbacksSM.connect('changed', on_changed)
+    mappable.callbacksSM.connect('changed', on_changed)
     mappable.colorbar = cb
     ax.figure.sca(ax)
     return cb
