@@ -7,7 +7,6 @@ Demo Colorbar of Inset Axes
 import matplotlib.pyplot as plt
 
 from mpl_toolkits.axes_grid1.inset_locator import inset_axes, zoomed_inset_axes
-from mpl_toolkits.axes_grid1.colorbar import colorbar
 
 
 def get_demo_image():
@@ -46,6 +45,6 @@ cax = inset_axes(axins,
                  borderpad=0,
                  )
 
-colorbar(im, cax=cax)
+fig.colorbar(im, cax=cax)
 
 plt.show()
