@@ -48,18 +48,18 @@ def plot_color_gradients(cmap_category, cmap_list):
     # Create figure and adjust figure height to number of colormaps
     nrows = len(cmap_list)
     figh = 0.35 + 0.15 + (nrows + (nrows-1)*0.1)*0.22
-    fig, axes = plt.subplots(nrows=nrows, figsize=(6.4, figh))
+    fig, axs = plt.subplots(nrows=nrows, figsize=(6.4, figh))
     fig.subplots_adjust(top=1-.35/figh, bottom=.15/figh, left=0.2, right=0.99)
 
-    axes[0].set_title(cmap_category + ' colormaps', fontsize=14)
+    axs[0].set_title(cmap_category + ' colormaps', fontsize=14)
 
-    for ax, name in zip(axes, cmap_list):
+    for ax, name in zip(axs, cmap_list):
         ax.imshow(gradient, aspect='auto', cmap=plt.get_cmap(name))
         ax.text(-.01, .5, name, va='center', ha='right', fontsize=10,
                 transform=ax.transAxes)
 
     # Turn off *all* ticks & spines, not just the ones with colormaps.
-    for ax in axes:
+    for ax in axs:
         ax.set_axis_off()
 
 
