@@ -7,8 +7,8 @@ This is a demo of creating a pdf file with several pages,
 as well as adding metadata and annotations to pdf files.
 
 If you want to use a multipage pdf file using LaTeX, you need
-to use `from matplotlib.backends.backend_pgf import PdfPages`.
-This version however does not support `attach_note`.
+to use ``from matplotlib.backends.backend_pgf import PdfPages``.
+This version however does not support `.attach_note`.
 """
 
 import datetime
@@ -32,8 +32,7 @@ with PdfPages('multipage_pdf.pdf') as pdf:
     x = np.arange(0, 5, 0.1)
     plt.plot(x, np.sin(x), 'b-')
     plt.title('Page Two')
-    pdf.attach_note("plot of sin(x)")  # you can add a pdf note to
-                                       # attach metadata to a page
+    pdf.attach_note("plot of sin(x)")  # attach metadata (as pdf note) to page
     pdf.savefig()
     plt.close()
 
