@@ -6,7 +6,6 @@ Create 3D histogram of 2D data
 Demo of a histogram for 2 dimensional data as a bar graph in 3D.
 """
 
-from mpl_toolkits.mplot3d import Axes3D
 import matplotlib.pyplot as plt
 import numpy as np
 
@@ -20,19 +19,15 @@ x, y = np.random.rand(2, 100) * 4
 hist, xedges, yedges = np.histogram2d(x, y, bins=4, range=[[0, 4], [0, 4]])
 
 # Construct arrays for the anchor positions of the 16 bars.
-# Note: np.meshgrid gives arrays in (ny, nx) so we use 'F' to flatten xpos,
-# ypos in column-major order. For numpy >= 1.7, we could instead call meshgrid
-# with indexing='ij'.
-xpos, ypos = np.meshgrid(xedges[:-1] + 0.25, yedges[:-1] + 0.25)
-xpos = xpos.flatten('F')
-ypos = ypos.flatten('F')
-zpos = np.zeros_like(xpos)
+xpos, ypos = np.meshgrid(xedges[:-1] + 0.25, yedges[:-1] + 0.25, indexing="ij")
+xpos = xpos.ravel()
+ypos = ypos.ravel()
+zpos = 0
 
 # Construct arrays with the dimensions for the 16 bars.
-dx = 0.5 * np.ones_like(zpos)
-dy = dx.copy()
-dz = hist.flatten()
+dx = dy = 0.5 * np.ones_like(zpos)
+dz = hist.ravel()
 
-ax.bar3d(xpos, ypos, zpos, dx, dy, dz, color='b', zsort='average')
+ax.bar3d(xpos, ypos, zpos, dx, dy, dz, zsort='average')
 
 plt.show()
