@@ -6,9 +6,6 @@ Triangular 3D surfaces
 Plot a 3D surface with a triangular mesh.
 '''
 
-# This import registers the 3D projection, but is otherwise unused.
-from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
-
 import matplotlib.pyplot as plt
 import numpy as np
 
