@@ -19,7 +19,7 @@ ax.set_xlim(0, 10)
 ax.set_ylim(-1, 1)
 
 xdata, ydata = 5, 0
-xdisplay, ydisplay = ax.transData.transform_point((xdata, ydata))
+xdisplay, ydisplay = ax.transData.transform((xdata, ydata))
 
 bbox = dict(boxstyle="round", fc="0.8")
 arrowprops = dict(
@@ -52,6 +52,6 @@ plt.show()
 # in this example:
 
 import matplotlib
-matplotlib.transforms.Transform.transform_point
+matplotlib.transforms.Transform.transform
 matplotlib.axes.Axes.annotate
 matplotlib.pyplot.annotate
