@@ -32,7 +32,7 @@ class MyFormatter(Formatter):
 
     def __call__(self, x, pos=0):
         'Return the label for time x at position pos'
-        ind = int(np.round(x))
+        ind = int(round(x))
         if ind >= len(self.dates) or ind < 0:
             return ''
         return dates.num2date(self.dates[ind]).strftime(self.fmt)
