@@ -12,7 +12,7 @@ x = [1, 2, 3, 4]
 y = [1, 4, 9, 6]
 labels = ['Frogs', 'Hogs', 'Bogs', 'Slogs']
 
-plt.plot(x, y, 'ro')
+plt.plot(x, y)
 # You can specify a rotation for the tick labels in degrees or with keywords.
 plt.xticks(x, labels, rotation='vertical')
 # Pad margins so that markers don't get clipped by the axes
