@@ -3,9 +3,10 @@
 Connect Simple01
 ================
 
-A `ConnectionPatch` can be used to draw a line (possibly with arrow head)
+A `.ConnectionPatch` can be used to draw a line (possibly with arrow head)
 between points defined in different coordinate systems and/or axes.
 """
+
 from matplotlib.patches import ConnectionPatch
 import matplotlib.pyplot as plt
 
@@ -26,21 +27,20 @@ ax1.add_artist(con)
 # Draw an arrow between the same point in data coordinates,
 # but in different axes.
 xy = (0.3, 0.2)
-coordsA = "data"
-coordsB = "data"
-con = ConnectionPatch(xyA=xy, xyB=xy, coordsA=coordsA, coordsB=coordsB,
-                      axesA=ax2, axesB=ax1,
-                      arrowstyle="->", shrinkB=5)
+con = ConnectionPatch(
+    xyA=xy, coordsA=ax2.transData,
+    xyB=xy, coordsB=ax1.transData,
+    arrowstyle="->", shrinkB=5)
 ax2.add_artist(con)
 
 # Draw a line between the different points, defined in different coordinate
 # systems.
-xyA = (0.6, 1.0)  # in axes coordinates
-xyB = (0.0, 0.2)  # x in axes coordinates, y in data coordinates
-coordsA = "axes fraction"
-coordsB = ax2.get_yaxis_transform()
-con = ConnectionPatch(xyA=xyA, xyB=xyB, coordsA=coordsA, coordsB=coordsB,
-                      arrowstyle="-")
+con = ConnectionPatch(
+    # in axes coordinates
+    xyA=(0.6, 1.0), coordsA=ax2.transAxes,
+    # x in axes coordinates, y in data coordinates
+    xyB=(0.0, 0.2), coordsB=ax2.get_yaxis_transform(),
+    arrowstyle="-")
 ax2.add_artist(con)
 
 ax1.set_xlim(0, 1)
